@@ -2,9 +2,13 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <malloc.h>
 #include "defines.h"
 
+#if defined(__alpha__)
+#include <malloc.h>
+#endif
+
+
 VERTEX_PTR *vp ;
 struct VERTEX *va ;
 EDGE_PTR *next ;
@@ -161,7 +165,7 @@ QUAD_EDGE next_edge, avail_edge;
 
 void delete_all_edges() { next_edge= 0; avail_edge = NYL;}
 
-#if defined(__POWERPC__) || defined(__FreeBSD__)
+#if defined(__APPLE__) || defined(__FreeBSD__)
 #define MEMALIGN_IS_NOT_AVAILABLE
 #endif
 
