@@ -1,10 +1,10 @@
 //===- DSGraph.h - Represent a collection of data structures ----*- C++ -*-===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 //
 // This header defines the data structure graph (DSGraph) and the
@@ -16,20 +16,23 @@
 #define LLVM_ANALYSIS_DSGRAPH_H
 
 #include "llvm/Analysis/DataStructure/DSNode.h"
+#include "llvm/ADT/hash_map"
+#include "llvm/ADT/EquivalenceClasses.h"
+#include <list>
 
 namespace llvm {
 
 class GlobalValue;
 
 //===----------------------------------------------------------------------===//
-/// DSScalarMap - An instance of this class is used to keep track of all of 
+/// DSScalarMap - An instance of this class is used to keep track of all of
 /// which DSNode each scalar in a function points to.  This is specialized to
-/// keep track of globals with nodes in the function, and to keep track of the 
+/// keep track of globals with nodes in the function, and to keep track of the
 /// unique DSNodeHandle being used by the scalar map.
 ///
-/// This class is crucial to the efficiency of DSA with some large SCC's.  In 
+/// This class is crucial to the efficiency of DSA with some large SCC's.  In
 /// these cases, the cost of iterating over the scalar map dominates the cost
-/// of DSA.  In all of these cases, the DSA phase is really trying to identify 
+/// of DSA.  In all of these cases, the DSA phase is really trying to identify
 /// globals or unique node handles active in the function.
 ///
 class DSScalarMap {
@@ -38,9 +41,14 @@ class DSScalarMap {
 
   typedef hash_set<GlobalValue*> GlobalSetTy;
   GlobalSetTy GlobalSet;
-public:
 
-  // Compatibility methods: provide an interface compatible with a map of 
+  EquivalenceClasses<GlobalValue*> &GlobalECs;
+public:
+  DSScalarMap(EquivalenceClasses<GlobalValue*> &ECs) : GlobalECs(ECs) {}
+
+  EquivalenceClasses<GlobalValue*> &getGlobalECs() const { return GlobalECs; }
+
+  // Compatibility methods: provide an interface compatible with a map of
   // Value* to DSNodeHandle's.
   typedef ValueMapTy::const_iterator const_iterator;
   typedef ValueMapTy::iterator iterator;
@@ -48,11 +56,60 @@ public:
   iterator end()   { return ValueMap.end(); }
   const_iterator begin() const { return ValueMap.begin(); }
   const_iterator end() const { return ValueMap.end(); }
-  iterator find(Value *V) { return ValueMap.find(V); }
-  const_iterator find(Value *V) const { return ValueMap.find(V); }
-  unsigned count(Value *V) const { return ValueMap.count(V); }
 
-  void erase(Value *V) { erase(find(V)); }
+  GlobalValue *getLeaderForGlobal(GlobalValue *GV) const {
+    EquivalenceClasses<GlobalValue*>::iterator ECI = GlobalECs.findValue(GV);
+    if (ECI == GlobalECs.end()) return GV;
+    return *GlobalECs.findLeader(ECI);
+  }
+
+
+  iterator find(Value *V) {
+    iterator I = ValueMap.find(V);
+    if (I != ValueMap.end()) return I;
+
+    if (GlobalValue *GV = dyn_cast<GlobalValue>(V)) {
+      // If this is a global, check to see if it is equivalenced to something
+      // in the map.
+      GlobalValue *Leader = getLeaderForGlobal(GV);
+      if (Leader != GV)
+        I = ValueMap.find((Value*)Leader);
+    }
+    return I;
+  }
+  const_iterator find(Value *V) const {
+    const_iterator I = ValueMap.find(V);
+    if (I != ValueMap.end()) return I;
+
+    if (GlobalValue *GV = dyn_cast<GlobalValue>(V)) {
+      // If this is a global, check to see if it is equivalenced to something
+      // in the map.
+      GlobalValue *Leader = getLeaderForGlobal(GV);
+      if (Leader != GV)
+        I = ValueMap.find((Value*)Leader);
+    }
+    return I;
+  }
+
+  /// getRawEntryRef - This method can be used by clients that are aware of the
+  /// global value equivalence class in effect.
+  DSNodeHandle &getRawEntryRef(Value *V) {
+    std::pair<iterator,bool> IP =
+      ValueMap.insert(std::make_pair(V, DSNodeHandle()));
+     if (IP.second)   // Inserted the new entry into the map.
+       if (GlobalValue *GV = dyn_cast<GlobalValue>(V))
+         GlobalSet.insert(GV);
+     return IP.first->second;
+  }
+
+  unsigned count(Value *V) const { return ValueMap.find(V) != ValueMap.end(); }
+
+  void erase(Value *V) { erase(ValueMap.find(V)); }
+
+  void eraseIfExists(Value *V) {
+    iterator I = find(V);
+    if (I != end()) erase(I);
+  }
 
   /// replaceScalar - When an instruction needs to be modified, this method can
   /// be used to update the scalar map to remove the old and insert the new.
@@ -64,21 +121,32 @@ public:
     erase(I);
   }
 
-  DSNodeHandle &operator[](Value *V) {
-    std::pair<iterator,bool> IP = 
-      ValueMap.insert(std::make_pair(V, DSNodeHandle()));
-    if (IP.second) {  // Inserted the new entry into the map.
-      if (GlobalValue *GV = dyn_cast<GlobalValue>(V))
-        GlobalSet.insert(GV);
-    }
-    return IP.first->second;
+  /// copyScalarIfExists - If Old exists in the scalar map, make New point to
+  /// whatever Old did.
+  void copyScalarIfExists(Value *Old, Value *New) {
+    iterator I = find(Old);
+    if (I != end())
+      ValueMap.insert(std::make_pair(New, I->second));
   }
 
-  void erase(iterator I) { 
+  /// operator[] - Return the DSNodeHandle for the specified value, creating a
+  /// new null handle if there is no entry yet.
+  DSNodeHandle &operator[](Value *V) {
+    iterator I = ValueMap.find(V);
+    if (I != ValueMap.end())
+      return I->second;   // Return value if already exists.
+
+    if (GlobalValue *GV = dyn_cast<GlobalValue>(V))
+      return AddGlobal(GV);
+
+    return ValueMap.insert(std::make_pair(V, DSNodeHandle())).first->second;
+  }
+
+  void erase(iterator I) {
     assert(I != ValueMap.end() && "Cannot erase end!");
     if (GlobalValue *GV = dyn_cast<GlobalValue>(I->first))
       GlobalSet.erase(GV);
-    ValueMap.erase(I); 
+    ValueMap.erase(I);
   }
 
   void clear() {
@@ -86,28 +154,40 @@ public:
     GlobalSet.clear();
   }
 
+  /// spliceFrom - Copy all entries from RHS, then clear RHS.
+  ///
+  void spliceFrom(DSScalarMap &RHS);
+
   // Access to the global set: the set of all globals currently in the
   // scalar map.
   typedef GlobalSetTy::const_iterator global_iterator;
   global_iterator global_begin() const { return GlobalSet.begin(); }
   global_iterator global_end() const { return GlobalSet.end(); }
+  unsigned global_size() const { return GlobalSet.size(); }
+  unsigned global_count(GlobalValue *GV) const { return GlobalSet.count(GV); }
+private:
+  DSNodeHandle &AddGlobal(GlobalValue *GV);
 };
 
 
 //===----------------------------------------------------------------------===//
 /// DSGraph - The graph that represents a function.
 ///
-struct DSGraph {
+class DSGraph {
+public:
   // Public data-type declarations...
   typedef DSScalarMap ScalarMapTy;
   typedef hash_map<Function*, DSNodeHandle> ReturnNodesTy;
-  typedef hash_set<GlobalValue*> GlobalSetTy;
   typedef ilist<DSNode> NodeListTy;
 
   /// NodeMapTy - This data type is used when cloning one graph into another to
   /// keep track of the correspondence between the nodes in the old and new
   /// graphs.
   typedef hash_map<const DSNode*, DSNodeHandle> NodeMapTy;
+
+  // InvNodeMapTy - This data type is used to represent the inverse of a node
+  // map.
+  typedef hash_multimap<DSNodeHandle, const DSNode*> InvNodeMapTy;
 private:
   DSGraph *GlobalsGraph;   // Pointer to the common graph of global objects
   bool PrintAuxCalls;      // Should this graph print the Aux calls vector?
@@ -121,38 +201,34 @@ private:
   //
   ReturnNodesTy ReturnNodes;
 
-  // FunctionCalls - This vector maintains a single entry for each call
+  // FunctionCalls - This list maintains a single entry for each call
   // instruction in the current graph.  The first entry in the vector is the
   // scalar that holds the return value for the call, the second is the function
   // scalar being invoked, and the rest are pointer arguments to the function.
   // This vector is built by the Local graph and is never modified after that.
   //
-  std::vector<DSCallSite> FunctionCalls;
+  std::list<DSCallSite> FunctionCalls;
 
   // AuxFunctionCalls - This vector contains call sites that have been processed
   // by some mechanism.  In pratice, the BU Analysis uses this vector to hold
   // the _unresolved_ call sites, because it cannot modify FunctionCalls.
   //
-  std::vector<DSCallSite> AuxFunctionCalls;
-
-  // InlinedGlobals - This set records which globals have been inlined from
-  // other graphs (callers or callees, depending on the pass) into this one.
-  // 
-  GlobalSetTy InlinedGlobals;
+  std::list<DSCallSite> AuxFunctionCalls;
 
   /// TD - This is the target data object for the machine this graph is
   /// constructed for.
   const TargetData &TD;
 
   void operator=(const DSGraph &); // DO NOT IMPLEMENT
-
+  DSGraph(const DSGraph&);         // DO NOT IMPLEMENT
 public:
   // Create a new, empty, DSGraph.
-  DSGraph(const TargetData &td)
-    : GlobalsGraph(0), PrintAuxCalls(false), TD(td) {}
+  DSGraph(EquivalenceClasses<GlobalValue*> &ECs, const TargetData &td)
+    : GlobalsGraph(0), PrintAuxCalls(false), ScalarMap(ECs), TD(td) {}
 
   // Compute the local DSGraph
-  DSGraph(const TargetData &td, Function &F, DSGraph *GlobalsGraph);
+  DSGraph(EquivalenceClasses<GlobalValue*> &ECs, const TargetData &TD,
+          Function &F, DSGraph *GlobalsGraph);
 
   // Copy ctor - If you want to capture the node mapping between the source and
   // destination graph, you may optionally do this by specifying a map to record
@@ -162,13 +238,19 @@ public:
   // source.  You need to set a new GlobalsGraph with the setGlobalsGraph
   // method.
   //
-  DSGraph(const DSGraph &DSG);
-  DSGraph(const DSGraph &DSG, NodeMapTy &NodeMap);
+  DSGraph(const DSGraph &DSG, EquivalenceClasses<GlobalValue*> &ECs,
+          unsigned CloneFlags = 0);
   ~DSGraph();
 
   DSGraph *getGlobalsGraph() const { return GlobalsGraph; }
   void setGlobalsGraph(DSGraph *G) { GlobalsGraph = G; }
 
+  /// getGlobalECs - Return the set of equivalence classes that the global
+  /// variables in the program form.
+  EquivalenceClasses<GlobalValue*> &getGlobalECs() const {
+    return ScalarMap.getGlobalECs();
+  }
+
   /// getTargetData - Return the TargetData object for the current target.
   ///
   const TargetData &getTargetData() const { return TD; }
@@ -184,9 +266,13 @@ public:
   /// cause the node to be removed from this list.  This means that if you are
   /// iterating over nodes and doing something that could cause _any_ node to
   /// merge, your node_iterators into this graph can be invalidated.
-  typedef NodeListTy::compat_iterator node_iterator;
-  node_iterator node_begin() const { return Nodes.compat_begin(); }
-  node_iterator node_end()   const { return Nodes.compat_end(); }
+  typedef NodeListTy::iterator node_iterator;
+  node_iterator node_begin() { return Nodes.begin(); }
+  node_iterator node_end()   { return Nodes.end(); }
+
+  typedef NodeListTy::const_iterator node_const_iterator;
+  node_const_iterator node_begin() const { return Nodes.begin(); }
+  node_const_iterator node_end()   const { return Nodes.end(); }
 
   /// getFunctionNames - Return a space separated list of the name of the
   /// functions in this graph (if any)
@@ -207,26 +293,27 @@ public:
   /// getFunctionCalls - Return the list of call sites in the original local
   /// graph...
   ///
-  const std::vector<DSCallSite> &getFunctionCalls() const {
-    return FunctionCalls;
-  }
+  const std::list<DSCallSite> &getFunctionCalls() const { return FunctionCalls;}
+  std::list<DSCallSite> &getFunctionCalls() { return FunctionCalls;}
 
   /// getAuxFunctionCalls - Get the call sites as modified by whatever passes
   /// have been run.
   ///
-  std::vector<DSCallSite> &getAuxFunctionCalls() {
-    return AuxFunctionCalls;
-  }
-  const std::vector<DSCallSite> &getAuxFunctionCalls() const {
+  std::list<DSCallSite> &getAuxFunctionCalls() { return AuxFunctionCalls; }
+  const std::list<DSCallSite> &getAuxFunctionCalls() const {
     return AuxFunctionCalls;
   }
 
-  /// getInlinedGlobals - Get the set of globals that are have been inlined
-  /// (from callees in BU or from callers in TD) into the current graph.
-  ///
-  GlobalSetTy& getInlinedGlobals() {
-    return InlinedGlobals;
-  }
+  // Function Call iteration
+  typedef std::list<DSCallSite>::const_iterator fc_iterator;
+  fc_iterator fc_begin() const { return FunctionCalls.begin(); }
+  fc_iterator fc_end() const { return FunctionCalls.end(); }
+
+
+  // Aux Function Call iteration
+  typedef std::list<DSCallSite>::const_iterator afc_iterator;
+  afc_iterator afc_begin() const { return AuxFunctionCalls.begin(); }
+  afc_iterator afc_end() const { return AuxFunctionCalls.end(); }
 
   /// getNodeForValue - Given a value that is used or defined in the body of the
   /// current function, return the DSNode that it points to.
@@ -240,6 +327,13 @@ public:
     return I->second;
   }
 
+  /// retnodes_* iterator methods: expose iteration over return nodes in the
+  /// graph, which are also the set of functions incorporated in this graph.
+  typedef ReturnNodesTy::const_iterator retnodes_iterator;
+  retnodes_iterator retnodes_begin() const { return ReturnNodes.begin(); }
+  retnodes_iterator retnodes_end() const { return ReturnNodes.end(); }
+
+
   /// getReturnNodes - Return the mapping of functions to their return nodes for
   /// this graph.
   ///
@@ -260,12 +354,28 @@ public:
     return I->second;
   }
 
+  /// containsFunction - Return true if this DSGraph contains information for
+  /// the specified function.
+  bool containsFunction(Function *F) const {
+    return ReturnNodes.count(F);
+  }
+
   /// getGraphSize - Return the number of nodes in this graph.
   ///
   unsigned getGraphSize() const {
     return Nodes.size();
   }
 
+  /// addObjectToGraph - This method can be used to add global, stack, and heap
+  /// objects to the graph.  This can be used when updating DSGraphs due to the
+  /// introduction of new temporary objects.  The new object is not pointed to
+  /// and does not point to any other objects in the graph.  Note that this
+  /// method initializes the type of the DSNode to the declared type of the
+  /// object if UseDeclaredType is true, otherwise it leaves the node type as
+  /// void.
+  DSNode *addObjectToGraph(Value *Ptr, bool UseDeclaredType = true);
+
+
   /// print - Print a dot graph to the specified ostream...
   ///
   void print(std::ostream &O) const;
@@ -286,7 +396,7 @@ public:
   ///
   void maskNodeTypes(unsigned Mask) {
     for (node_iterator I = node_begin(), E = node_end(); I != E; ++I)
-      (*I)->maskNodeTypes(Mask);
+      I->maskNodeTypes(Mask);
   }
   void maskIncompleteMarkers() { maskNodeTypes(~DSNode::Incomplete); }
 
@@ -323,7 +433,6 @@ public:
     DontCloneAuxCallNodes = 1 << 2, CloneAuxCallNodes = 0,
     StripModRefBits       = 1 << 3, KeepModRefBits    = 0,
     StripIncompleteBit    = 1 << 4, KeepIncompleteBit = 0,
-    UpdateInlinedGlobals  = 1 << 5, DontUpdateInlinedGlobals = 0,
   };
 
   void updateFromGlobalGraph();
@@ -336,24 +445,51 @@ public:
                                  const DSNodeHandle &NH2, NodeMapTy &NodeMap,
                                  bool StrictChecking = true);
 
+  /// computeGToGGMapping - Compute the mapping of nodes in the graph to nodes
+  /// in the globals graph.
+  void computeGToGGMapping(NodeMapTy &NodeMap);
 
-  /// cloneInto - Clone the specified DSGraph into the current graph.  The
-  /// translated ScalarMap for the old function is filled into the OldValMap
-  /// member, and the translated ReturnNodes map is returned into ReturnNodes.
-  /// OldNodeMap contains a mapping from the original nodes to the newly cloned
-  /// nodes.
+  /// computeGGToGMapping - Compute the mapping of nodes in the global
+  /// graph to nodes in this graph.
+  void computeGGToGMapping(InvNodeMapTy &InvNodeMap);
+
+  /// computeCalleeCallerMapping - Given a call from a function in the current
+  /// graph to the 'Callee' function (which lives in 'CalleeGraph'), compute the
+  /// mapping of nodes from the callee to nodes in the caller.
+  void computeCalleeCallerMapping(DSCallSite CS, const Function &Callee,
+                                  DSGraph &CalleeGraph, NodeMapTy &NodeMap);
+
+  /// spliceFrom - Logically perform the operation of cloning the RHS graph into
+  /// this graph, then clearing the RHS graph.  Instead of performing this as
+  /// two seperate operations, do it as a single, much faster, one.
+  ///
+  void spliceFrom(DSGraph &RHS);
+
+  /// cloneInto - Clone the specified DSGraph into the current graph.
   ///
   /// The CloneFlags member controls various aspects of the cloning process.
   ///
-  void cloneInto(const DSGraph &G, ScalarMapTy &OldValMap,
-                 ReturnNodesTy &OldReturnNodes, NodeMapTy &OldNodeMap,
-                 unsigned CloneFlags = 0);
+  void cloneInto(const DSGraph &G, unsigned CloneFlags = 0);
 
-  /// mergeInGraph - The method is used for merging graphs together.  If the
-  /// argument graph is not *this, it makes a clone of the specified graph, then
-  /// merges the nodes specified in the call site with the formal arguments in
-  /// the graph.  If the StripAlloca's argument is 'StripAllocaBit' then Alloca
-  /// markers are removed from nodes.
+  /// getFunctionArgumentsForCall - Given a function that is currently in this
+  /// graph, return the DSNodeHandles that correspond to the pointer-compatible
+  /// function arguments.  The vector is filled in with the return value (or
+  /// null if it is not pointer compatible), followed by all of the
+  /// pointer-compatible arguments.
+  void getFunctionArgumentsForCall(Function *F,
+                                   std::vector<DSNodeHandle> &Args) const;
+
+  /// mergeInGraph - This graph merges in the minimal number of
+  /// nodes from G2 into 'this' graph, merging the bindings specified by the
+  /// call site (in this graph) with the bindings specified by the vector in G2.
+  /// If the StripAlloca's argument is 'StripAllocaBit' then Alloca markers are
+  /// removed from nodes.
+  ///
+  void mergeInGraph(const DSCallSite &CS, std::vector<DSNodeHandle> &Args,
+                    const DSGraph &G2, unsigned CloneFlags);
+
+  /// mergeInGraph - This method is the same as the above method, but the
+  /// argument bindings are provided by using the formal arguments of F.
   ///
   void mergeInGraph(const DSCallSite &CS, Function &F, const DSGraph &Graph,
                     unsigned CloneFlags);
@@ -372,10 +508,7 @@ public:
     assert((!N || N->getParentGraph() == this) &&
            "AssertNodeInGraph: Node is not in graph!");
   }
-  void AssertNodeContainsGlobal(const DSNode *N, GlobalValue *GV) const {
-    assert(std::find(N->getGlobals().begin(), N->getGlobals().end(), GV) !=
-           N->getGlobals().end() && "Global value not in node!");
-  }
+  void AssertNodeContainsGlobal(const DSNode *N, GlobalValue *GV) const;
 
   void AssertCallSiteInGraph(const DSCallSite &CS) const;
   void AssertCallNodesInGraph() const;
@@ -422,7 +555,7 @@ public:
     if (CloneFlags & DSGraph::StripIncompleteBit)
       BitsToKeep &= ~DSNode::Incomplete;
   }
-  
+
   DSNodeHandle getClonedNH(const DSNodeHandle &SrcNH);
 
   void merge(const DSNodeHandle &NH, const DSNodeHandle &SrcNH);
@@ -430,7 +563,7 @@ public:
   /// mergeCallSite - Merge the nodes reachable from the specified src call
   /// site into the nodes reachable from DestCS.
   ///
-  void mergeCallSite(const DSCallSite &DestCS, const DSCallSite &SrcCS);
+  void mergeCallSite(DSCallSite &DestCS, const DSCallSite &SrcCS);
 
   bool clonedAnyNodes() const { return !NodeMap.empty(); }
 
