@@ -1,10 +1,10 @@
 //===- Transforms/Instrumentation.h - Instrumentation passes ----*- C++ -*-===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 //
 // This files defines constructor functions for instrumentation passes.
@@ -20,14 +20,28 @@ class ModulePass;
 class FunctionPass;
 
 // Reoptimizer support pass: add instrumentation calls to back-edges of loops
-ModulePass *createLoopInstrumentationPass ();
+FunctionPass *createLoopInstrumentationPass ();
 
 // Reoptimizer support pass: combine multiple back-edges w/ same target into one
 FunctionPass *createCombineBranchesPass();
 
 // Reoptimizer support pass: emit table of global functions
-FunctionPass *createEmitFunctionTablePass ();
+ModulePass *createEmitFunctionTablePass ();
 
+// Reoptimizer support pass: insert function profiling instrumentation
+ModulePass *createFunctionProfilerPass();
+
+// Reoptimizer support pass: insert block profiling instrumentation
+ModulePass *createBlockProfilerPass();
+
+// Reoptimizer support pass: insert edge profiling instrumentation
+ModulePass *createEdgeProfilerPass();
+
+// Reoptimizer support pass: insert basic block tracing instrumentation
+ModulePass *createTraceBasicBlockPass();
+
+// Reoptimizer support pass: insert counting of execute paths instrumentation
+FunctionPass *createProfilePathsPass();
 
 //===----------------------------------------------------------------------===//
 // Support for inserting LLVM code to print values at basic block and function
