@@ -1,10 +1,10 @@
 //===- Local.cpp - Compute a local data structure graph for a function ----===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 //
 // Compute the local version of the data structure graph for a function.  The
@@ -73,16 +73,17 @@ namespace {
     DSGraph &G;
     DSNodeHandle *RetNode;               // Node that gets returned...
     DSScalarMap &ScalarMap;
-    std::vector<DSCallSite> *FunctionCalls;
+    std::list<DSCallSite> *FunctionCalls;
 
   public:
-    GraphBuilder(Function &f, DSGraph &g, DSNodeHandle &retNode, 
-                 std::vector<DSCallSite> &fc)
+    GraphBuilder(Function &f, DSGraph &g, DSNodeHandle &retNode,
+                 std::list<DSCallSite> &fc)
       : G(g), RetNode(&retNode), ScalarMap(G.getScalarMap()),
         FunctionCalls(&fc) {
 
       // Create scalar nodes for all pointer arguments...
-      for (Function::aiterator I = f.abegin(), E = f.aend(); I != E; ++I)
+      for (Function::arg_iterator I = f.arg_begin(), E = f.arg_end();
+           I != E; ++I)
         if (isPointerType(I->getType()))
           getValueDest(*I);
 
@@ -104,6 +105,7 @@ namespace {
     void handleAlloc(AllocationInst &AI, bool isHeap);
 
     void visitPHINode(PHINode &PN);
+    void visitSelectInst(SelectInst &SI);
 
     void visitGetElementPtrInst(User &GEP);
     void visitReturnInst(ReturnInst &RI);
@@ -111,13 +113,12 @@ namespace {
     void visitStoreInst(StoreInst &SI);
     void visitCallInst(CallInst &CI);
     void visitInvokeInst(InvokeInst &II);
-    void visitSetCondInst(SetCondInst &SCI) {}  // SetEQ & friends are ignored
+    void visitSetCondInst(SetCondInst &SCI);
     void visitFreeInst(FreeInst &FI);
     void visitCastInst(CastInst &CI);
     void visitInstruction(Instruction &I);
 
     void visitCallSite(CallSite CS);
-    void visitVANextInst(VANextInst &I);
     void visitVAArgInst(VAArgInst   &I);
 
     void MergeConstantInitIntoNode(DSNodeHandle &NH, Constant *C);
@@ -146,7 +147,7 @@ namespace {
     ///
     void setDestTo(Value &V, const DSNodeHandle &NH);
 
-    /// getValueDest - Return the DSNode that the actual value points to. 
+    /// getValueDest - Return the DSNode that the actual value points to.
     ///
     DSNodeHandle getValueDest(Value &V);
 
@@ -163,8 +164,9 @@ using namespace DS;
 //===----------------------------------------------------------------------===//
 // DSGraph constructor - Simply use the GraphBuilder to construct the local
 // graph.
-DSGraph::DSGraph(const TargetData &td, Function &F, DSGraph *GG)
-  : GlobalsGraph(GG), TD(td) {
+DSGraph::DSGraph(EquivalenceClasses<GlobalValue*> &ECs, const TargetData &td,
+                 Function &F, DSGraph *GG)
+  : GlobalsGraph(GG), ScalarMap(ECs), TD(td) {
   PrintAuxCalls = false;
 
   DEBUG(std::cerr << "  [Loc] Calculating graph for: " << F.getName() << "\n");
@@ -175,18 +177,11 @@ DSGraph::DSGraph(const TargetData &td, Function &F, DSGraph *GG)
   Timer::addPeakMemoryMeasurement();
 #endif
 
-  // Remove all integral constants from the scalarmap!
-  for (DSScalarMap::iterator I = ScalarMap.begin(); I != ScalarMap.end();)
-    if (isa<ConstantIntegral>(I->first))
-      ScalarMap.erase(I++);
-    else
-      ++I;
-
   // If there are any constant globals referenced in this function, merge their
   // initializers into the local graph from the globals graph.
   if (ScalarMap.global_begin() != ScalarMap.global_end()) {
     ReachabilityCloner RC(*this, *GG, 0);
-    
+
     for (DSScalarMap::global_iterator I = ScalarMap.global_begin();
          I != ScalarMap.global_end(); ++I)
       if (GlobalVariable *GV = dyn_cast<GlobalVariable>(*I))
@@ -221,14 +216,17 @@ DSNodeHandle GraphBuilder::getValueDest(Value &Val) {
   // extract the actual value.
   DSNode* N;
   if (GlobalValue* GV = dyn_cast<GlobalValue>(V)) {
-    // Create a new global node for this global variable...
+    // Create a new global node for this global variable.
     N = createNode(GV->getType()->getElementType());
     N->addGlobal(GV);
   } else if (Constant *C = dyn_cast<Constant>(V)) {
     if (ConstantExpr *CE = dyn_cast<ConstantExpr>(C)) {
-      if (CE->getOpcode() == Instruction::Cast)
-        NH = getValueDest(*CE->getOperand(0));
-      else if (CE->getOpcode() == Instruction::GetElementPtr) {
+      if (CE->getOpcode() == Instruction::Cast) {
+        if (isa<PointerType>(CE->getOperand(0)->getType()))
+          NH = getValueDest(*CE->getOperand(0));
+        else
+          NH = createNode()->setUnknownNodeMarker();
+      } else if (CE->getOpcode() == Instruction::GetElementPtr) {
         visitGetElementPtrInst(*CE);
         DSScalarMap::iterator I = ScalarMap.find(CE);
         assert(I != ScalarMap.end() && "GEP didn't get processed right?");
@@ -242,10 +240,6 @@ DSNodeHandle GraphBuilder::getValueDest(Value &Val) {
         return 0;
       }
       return NH;
-
-    } else if (ConstantIntegral *CI = dyn_cast<ConstantIntegral>(C)) {
-      // Random constants are unknown mem
-      return NH = createNode()->setUnknownNodeMarker();
     } else if (isa<UndefValue>(C)) {
       ScalarMap.erase(V);
       return 0;
@@ -316,9 +310,25 @@ void GraphBuilder::visitPHINode(PHINode &PN) {
     PNDest.mergeWith(getValueDest(*PN.getIncomingValue(i)));
 }
 
+void GraphBuilder::visitSelectInst(SelectInst &SI) {
+  if (!isPointerType(SI.getType())) return; // Only pointer Selects
+
+  DSNodeHandle &Dest = ScalarMap[&SI];
+  Dest.mergeWith(getValueDest(*SI.getOperand(1)));
+  Dest.mergeWith(getValueDest(*SI.getOperand(2)));
+}
+
+void GraphBuilder::visitSetCondInst(SetCondInst &SCI) {
+  if (!isPointerType(SCI.getOperand(0)->getType()) ||
+      isa<ConstantPointerNull>(SCI.getOperand(1))) return; // Only pointers
+  ScalarMap[SCI.getOperand(0)].mergeWith(getValueDest(*SCI.getOperand(1)));
+}
+
+
 void GraphBuilder::visitGetElementPtrInst(User &GEP) {
   DSNodeHandle Value = getValueDest(*GEP.getOperand(0));
-  if (Value.isNull()) return;
+  if (Value.isNull())
+    Value = createNode();
 
   // As a special case, if all of the index operands of GEP are constant zeros,
   // handle this just like we handle casts (ie, don't do much).
@@ -332,7 +342,8 @@ void GraphBuilder::visitGetElementPtrInst(User &GEP) {
 
   // If all of the indices are zero, the result points to the operand without
   // applying the type.
-  if (AllZeros) {
+  if (AllZeros || (!Value.isNull() &&
+                   Value.getNode()->isNodeCompletelyFolded())) {
     setDestTo(GEP, Value);
     return;
   }
@@ -382,8 +393,9 @@ void GraphBuilder::visitGetElementPtrInst(User &GEP) {
   for (gep_type_iterator I = gep_type_begin(GEP), E = gep_type_end(GEP);
        I != E; ++I)
     if (const StructType *STy = dyn_cast<StructType>(*I)) {
-      unsigned FieldNo = cast<ConstantUInt>(I.getOperand())->getValue();
-      Offset += TD.getStructLayout(STy)->MemberOffsets[FieldNo];
+      unsigned FieldNo =
+           (unsigned)cast<ConstantUInt>(I.getOperand())->getValue();
+      Offset += (unsigned)TD.getStructLayout(STy)->MemberOffsets[FieldNo];
     } else if (const PointerType *PTy = dyn_cast<PointerType>(*I)) {
       if (!isa<Constant>(I.getOperand()) ||
           !cast<Constant>(I.getOperand())->isNullValue())
@@ -428,7 +440,8 @@ void GraphBuilder::visitGetElementPtrInst(User &GEP) {
 
 void GraphBuilder::visitLoadInst(LoadInst &LI) {
   DSNodeHandle Ptr = getValueDest(*LI.getOperand(0));
-  if (Ptr.getNode() == 0) return;
+  if (Ptr.isNull())
+    Ptr = createNode();
 
   // Make that the node is read from...
   Ptr.getNode()->setReadMarker();
@@ -461,11 +474,8 @@ void GraphBuilder::visitReturnInst(ReturnInst &RI) {
     RetNode->mergeWith(getValueDest(*RI.getOperand(0)));
 }
 
-void GraphBuilder::visitVANextInst(VANextInst &I) {
-  getValueDest(*I.getOperand(0)).mergeWith(getValueDest(I));
-}
-
 void GraphBuilder::visitVAArgInst(VAArgInst &I) {
+  //FIXME: also updates the argument
   DSNodeHandle Ptr = getValueDest(*I.getOperand(0));
   if (Ptr.isNull()) return;
 
@@ -559,7 +569,7 @@ void GraphBuilder::visitCallSite(CallSite CS) {
                AI != E; ++AI) {
             if (isPointerType((*AI)->getType()))
               if (DSNode *N = getValueDest(**AI).getNode())
-                N->setReadMarker();   
+                N->setReadMarker();
           }
           return;
         } else if (F->getName() == "read" || F->getName() == "pipe" ||
@@ -569,7 +579,7 @@ void GraphBuilder::visitCallSite(CallSite CS) {
                AI != E; ++AI) {
             if (isPointerType((*AI)->getType()))
               if (DSNode *N = getValueDest(**AI).getNode())
-                N->setModifiedMarker();   
+                N->setModifiedMarker();
           }
           return;
         } else if (F->getName() == "stat" || F->getName() == "fstat" ||
@@ -616,7 +626,7 @@ void GraphBuilder::visitCallSite(CallSite CS) {
             if (isPointerType((*AI)->getType()))
               if (DSNode *N = getValueDest(**AI).getNode())
                 N->setReadMarker();
-          
+
           // fopen allocates in an unknown way and writes to the file
           // descriptor.  Also, merge the allocated type into the node.
           DSNodeHandle Result = getValueDest(*CS.getInstruction());
@@ -629,9 +639,12 @@ void GraphBuilder::visitCallSite(CallSite CS) {
 
           // If this is freopen, merge the file descriptor passed in with the
           // result.
-          if (F->getName() == "freopen")
-            Result.mergeWith(getValueDest(**--CS.arg_end()));
-
+          if (F->getName() == "freopen") {
+            // ICC doesn't handle getting the iterator, decrementing and
+            // dereferencing it in one operation without error. Do it in 2 steps
+            CallSite::arg_iterator compit = CS.arg_end();
+            Result.mergeWith(getValueDest(**--compit));
+          }
           return;
         } else if (F->getName() == "fclose" && CS.arg_end()-CS.arg_begin() ==1){
           // fclose reads and deallocates the memory in an unknown way for the
@@ -644,7 +657,7 @@ void GraphBuilder::visitCallSite(CallSite CS) {
               N->mergeTypeInfo(PTy->getElementType(), H.getOffset());
           }
           return;
-        } else if (CS.arg_end()-CS.arg_begin() == 1 && 
+        } else if (CS.arg_end()-CS.arg_begin() == 1 &&
                    (F->getName() == "fflush" || F->getName() == "feof" ||
                     F->getName() == "fileno" || F->getName() == "clearerr" ||
                     F->getName() == "rewind" || F->getName() == "ftell" ||
@@ -655,17 +668,18 @@ void GraphBuilder::visitCallSite(CallSite CS) {
           DSNodeHandle H = getValueDest(**CS.arg_begin());
           if (DSNode *N = H.getNode()) {
             N->setReadMarker()->setModifiedMarker();
-          
+
             const Type *ArgTy = F->getFunctionType()->getParamType(0);
             if (const PointerType *PTy = dyn_cast<PointerType>(ArgTy))
               N->mergeTypeInfo(PTy->getElementType(), H.getOffset());
           }
           return;
-        } else if (CS.arg_end()-CS.arg_begin() == 4 && 
+        } else if (CS.arg_end()-CS.arg_begin() == 4 &&
                    (F->getName() == "fwrite" || F->getName() == "fread")) {
           // fread writes the first operand, fwrite reads it.  They both
           // read/write the FILE descriptor, and merges the FILE type.
-          DSNodeHandle H = getValueDest(**--CS.arg_end());
+          CallSite::arg_iterator compit = CS.arg_end();
+          DSNodeHandle H = getValueDest(**--compit);
           if (DSNode *N = H.getNode()) {
             N->setReadMarker()->setModifiedMarker();
             const Type *ArgTy = F->getFunctionType()->getParamType(3);
@@ -706,10 +720,12 @@ void GraphBuilder::visitCallSite(CallSite CS) {
                    F->getName() == "_IO_putc") {
           // These functions read and write the memory for the file descriptor,
           // which is passes as the last argument.
-          DSNodeHandle H = getValueDest(**--CS.arg_end());
+          CallSite::arg_iterator compit = CS.arg_end();
+          DSNodeHandle H = getValueDest(**--compit);
           if (DSNode *N = H.getNode()) {
             N->setReadMarker()->setModifiedMarker();
-            const Type *ArgTy = *--F->getFunctionType()->param_end();
+            FunctionType::param_iterator compit2 = F->getFunctionType()->param_end();
+            const Type *ArgTy = *--compit2;
             if (const PointerType *PTy = dyn_cast<PointerType>(ArgTy))
               N->mergeTypeInfo(PTy->getElementType(), H.getOffset());
           }
@@ -719,7 +735,7 @@ void GraphBuilder::visitCallSite(CallSite CS) {
                AI != E; ++AI)
             if (isPointerType((*AI)->getType()))
               if (DSNode *N = getValueDest(**AI).getNode())
-                N->setReadMarker();   
+                N->setReadMarker();
           return;
         } else if (F->getName() == "fseek" || F->getName() == "fgetpos" ||
                    F->getName() == "fsetpos") {
@@ -727,7 +743,8 @@ void GraphBuilder::visitCallSite(CallSite CS) {
           // and read/write all other arguments.
           DSNodeHandle H = getValueDest(**CS.arg_begin());
           if (DSNode *N = H.getNode()) {
-            const Type *ArgTy = *--F->getFunctionType()->param_end();
+            FunctionType::param_iterator compit2 = F->getFunctionType()->param_end();
+            const Type *ArgTy = *--compit2;
             if (const PointerType *PTy = dyn_cast<PointerType>(ArgTy))
               N->mergeTypeInfo(PTy->getElementType(), H.getOffset());
           }
@@ -768,7 +785,7 @@ void GraphBuilder::visitCallSite(CallSite CS) {
             // printf reads all pointer arguments.
             if (isPointerType((*AI)->getType()))
               if (DSNode *N = getValueDest(**AI).getNode())
-                N->setReadMarker();   
+                N->setReadMarker();
           }
           return;
         } else if (F->getName() == "vprintf" || F->getName() == "vfprintf" ||
@@ -804,7 +821,7 @@ void GraphBuilder::visitCallSite(CallSite CS) {
                 N->setReadMarker();
             ++AI;
           }
-          
+
           // Read the valist, and the pointed-to objects.
           if (AI != E && isPointerType((*AI)->getType())) {
             const DSNodeHandle &VAList = getValueDest(**AI);
@@ -848,7 +865,7 @@ void GraphBuilder::visitCallSite(CallSite CS) {
             // scanf writes all pointer arguments.
             if (isPointerType((*AI)->getType()))
               if (DSNode *N = getValueDest(**AI).getNode())
-                N->setModifiedMarker();   
+                N->setModifiedMarker();
           }
           return;
         } else if (F->getName() == "strtok") {
@@ -884,7 +901,7 @@ void GraphBuilder::visitCallSite(CallSite CS) {
             if (isPointerType((*AI)->getType()))
               if (DSNode *N = getValueDest(**AI).getNode())
                 N->setReadMarker();
-    
+
           if (DSNode *N = H.getNode())
             N->setReadMarker();
           return;
@@ -897,7 +914,8 @@ void GraphBuilder::visitCallSite(CallSite CS) {
           return;
         } else if (F->getName() == "modf" && CS.arg_end()-CS.arg_begin() == 2) {
           // This writes its second argument, and forces it to double.
-          DSNodeHandle H = getValueDest(**--CS.arg_end());
+          CallSite::arg_iterator compit = CS.arg_end();
+          DSNodeHandle H = getValueDest(**--compit);
           if (DSNode *N = H.getNode()) {
             N->setModifiedMarker();
             N->mergeTypeInfo(Type::DoubleTy, H.getOffset());
@@ -1022,7 +1040,7 @@ void GraphBuilder::MergeConstantInitIntoNode(DSNodeHandle &NH, Constant *C) {
     const StructLayout *SL = TD.getStructLayout(CS->getType());
     for (unsigned i = 0, e = CS->getNumOperands(); i != e; ++i) {
       DSNode *NHN = NH.getNode();
-      DSNodeHandle NewNH(NHN, NH.getOffset()+SL->MemberOffsets[i]);
+      DSNodeHandle NewNH(NHN, NH.getOffset()+(unsigned)SL->MemberOffsets[i]);
       MergeConstantInitIntoNode(NewNH, cast<Constant>(CS->getOperand(i)));
     }
   } else if (isa<ConstantAggregateZero>(C) || isa<UndefValue>(C)) {
@@ -1040,27 +1058,131 @@ void GraphBuilder::mergeInGlobalInitializer(GlobalVariable *GV) {
 }
 
 
-bool LocalDataStructures::runOnModule(Module &M) {
-  GlobalsGraph = new DSGraph(getAnalysis<TargetData>());
+/// BuildGlobalECs - Look at all of the nodes in the globals graph.  If any node
+/// contains multiple globals, DSA will never, ever, be able to tell the globals
+/// apart.  Instead of maintaining this information in all of the graphs
+/// throughout the entire program, store only a single global (the "leader") in
+/// the graphs, and build equivalence classes for the rest of the globals.
+static void BuildGlobalECs(DSGraph &GG, std::set<GlobalValue*> &ECGlobals) {
+  DSScalarMap &SM = GG.getScalarMap();
+  EquivalenceClasses<GlobalValue*> &GlobalECs = SM.getGlobalECs();
+  for (DSGraph::node_iterator I = GG.node_begin(), E = GG.node_end();
+       I != E; ++I) {
+    if (I->getGlobalsList().size() <= 1) continue;
 
+    // First, build up the equivalence set for this block of globals.
+    const std::vector<GlobalValue*> &GVs = I->getGlobalsList();
+    GlobalValue *First = GVs[0];
+    for (unsigned i = 1, e = GVs.size(); i != e; ++i)
+      GlobalECs.unionSets(First, GVs[i]);
+
+    // Next, get the leader element.
+    assert(First == GlobalECs.getLeaderValue(First) &&
+           "First did not end up being the leader?");
+
+    // Next, remove all globals from the scalar map that are not the leader.
+    assert(GVs[0] == First && "First had to be at the front!");
+    for (unsigned i = 1, e = GVs.size(); i != e; ++i) {
+      ECGlobals.insert(GVs[i]);
+      SM.erase(SM.find(GVs[i]));
+    }
+
+    // Finally, change the global node to only contain the leader.
+    I->clearGlobals();
+    I->addGlobal(First);
+  }
+
+  DEBUG(GG.AssertGraphOK());
+}
+
+/// EliminateUsesOfECGlobals - Once we have determined that some globals are in
+/// really just equivalent to some other globals, remove the globals from the
+/// specified DSGraph (if present), and merge any nodes with their leader nodes.
+static void EliminateUsesOfECGlobals(DSGraph &G,
+                                     const std::set<GlobalValue*> &ECGlobals) {
+  DSScalarMap &SM = G.getScalarMap();
+  EquivalenceClasses<GlobalValue*> &GlobalECs = SM.getGlobalECs();
+
+  bool MadeChange = false;
+  for (DSScalarMap::global_iterator GI = SM.global_begin(), E = SM.global_end();
+       GI != E; ) {
+    GlobalValue *GV = *GI++;
+    if (!ECGlobals.count(GV)) continue;
+
+    const DSNodeHandle &GVNH = SM[GV];
+    assert(!GVNH.isNull() && "Global has null NH!?");
+
+    // Okay, this global is in some equivalence class.  Start by finding the
+    // leader of the class.
+    GlobalValue *Leader = GlobalECs.getLeaderValue(GV);
+
+    // If the leader isn't already in the graph, insert it into the node
+    // corresponding to GV.
+    if (!SM.global_count(Leader)) {
+      GVNH.getNode()->addGlobal(Leader);
+      SM[Leader] = GVNH;
+    } else {
+      // Otherwise, the leader is in the graph, make sure the nodes are the
+      // merged in the specified graph.
+      const DSNodeHandle &LNH = SM[Leader];
+      if (LNH.getNode() != GVNH.getNode())
+        LNH.mergeWith(GVNH);
+    }
+
+    // Next step, remove the global from the DSNode.
+    GVNH.getNode()->removeGlobal(GV);
+
+    // Finally, remove the global from the ScalarMap.
+    SM.erase(GV);
+    MadeChange = true;
+  }
+
+  DEBUG(if(MadeChange) G.AssertGraphOK());
+}
+
+bool LocalDataStructures::runOnModule(Module &M) {
   const TargetData &TD = getAnalysis<TargetData>();
 
+  // First step, build the globals graph.
+  GlobalsGraph = new DSGraph(GlobalECs, TD);
   {
     GraphBuilder GGB(*GlobalsGraph);
-    
-    // Add initializers for all of the globals to the globals graph...
-    for (Module::giterator I = M.gbegin(), E = M.gend(); I != E; ++I)
+
+    // Add initializers for all of the globals to the globals graph.
+    for (Module::global_iterator I = M.global_begin(), E = M.global_end();
+         I != E; ++I)
       if (!I->isExternal())
         GGB.mergeInGlobalInitializer(I);
   }
 
+  // Next step, iterate through the nodes in the globals graph, unioning
+  // together the globals into equivalence classes.
+  std::set<GlobalValue*> ECGlobals;
+  BuildGlobalECs(*GlobalsGraph, ECGlobals);
+  DEBUG(std::cerr << "Eliminating " << ECGlobals.size() << " EC Globals!\n");
+  ECGlobals.clear();
+
   // Calculate all of the graphs...
   for (Module::iterator I = M.begin(), E = M.end(); I != E; ++I)
     if (!I->isExternal())
-      DSInfo.insert(std::make_pair(I, new DSGraph(TD, *I, GlobalsGraph)));
+      DSInfo.insert(std::make_pair(I, new DSGraph(GlobalECs, TD, *I,
+                                                  GlobalsGraph)));
 
   GlobalsGraph->removeTriviallyDeadNodes();
   GlobalsGraph->markIncompleteNodes(DSGraph::MarkFormalArgs);
+
+  // Now that we've computed all of the graphs, and merged all of the info into
+  // the globals graph, see if we have further constrained the globals in the
+  // program if so, update GlobalECs and remove the extraneous globals from the
+  // program.
+  BuildGlobalECs(*GlobalsGraph, ECGlobals);
+  if (!ECGlobals.empty()) {
+    DEBUG(std::cerr << "Eliminating " << ECGlobals.size() << " EC Globals!\n");
+    for (hash_map<Function*, DSGraph*>::iterator I = DSInfo.begin(),
+           E = DSInfo.end(); I != E; ++I)
+      EliminateUsesOfECGlobals(*I->second, ECGlobals);
+  }
+
   return false;
 }
 
