@@ -1,12 +1,12 @@
 //===-- ExecutionEngine.cpp - Common Implementation shared by EEs ---------===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
-// 
+//
 // This file defines the common interface used by the various execution engine
 // subclasses.
 //
@@ -33,7 +33,7 @@ namespace {
   Statistic<> NumGlobals  ("lli", "Number of global vars initialized");
 }
 
-ExecutionEngine::ExecutionEngine(ModuleProvider *P) : 
+ExecutionEngine::ExecutionEngine(ModuleProvider *P) :
   CurMod(*P->getModule()), MP(P) {
   assert(P && "ModuleProvider is null?");
 }
@@ -50,16 +50,18 @@ ExecutionEngine::~ExecutionEngine() {
 /// at the specified address.
 ///
 const GlobalValue *ExecutionEngine::getGlobalValueAtAddress(void *Addr) {
+  MutexGuard locked(lock);
+
   // If we haven't computed the reverse mapping yet, do so first.
-  if (GlobalAddressReverseMap.empty()) {
-    for (std::map<const GlobalValue*, void *>::iterator I = 
-           GlobalAddressMap.begin(), E = GlobalAddressMap.end(); I != E; ++I)
-      GlobalAddressReverseMap.insert(std::make_pair(I->second, I->first));
+  if (state.getGlobalAddressReverseMap(locked).empty()) {
+    for (std::map<const GlobalValue*, void *>::iterator I =
+           state.getGlobalAddressMap(locked).begin(), E = state.getGlobalAddressMap(locked).end(); I != E; ++I)
+      state.getGlobalAddressReverseMap(locked).insert(std::make_pair(I->second, I->first));
   }
 
   std::map<void *, const GlobalValue*>::iterator I =
-    GlobalAddressReverseMap.find(Addr);
-  return I != GlobalAddressReverseMap.end() ? I->second : 0;
+    state.getGlobalAddressReverseMap(locked).find(Addr);
+  return I != state.getGlobalAddressReverseMap(locked).end() ? I->second : 0;
 }
 
 // CreateArgv - Turn a vector of strings into a nice argv style array of
@@ -77,10 +79,10 @@ static void *CreateArgv(ExecutionEngine *EE,
     unsigned Size = InputArgv[i].size()+1;
     char *Dest = new char[Size];
     DEBUG(std::cerr << "ARGV[" << i << "] = " << (void*)Dest << "\n");
-      
+
     std::copy(InputArgv[i].begin(), InputArgv[i].end(), Dest);
     Dest[Size-1] = 0;
-      
+
     // Endian safe: Result[i] = (PointerTy)Dest;
     EE->StoreValueToMemory(PTOGV(Dest), (GenericValue*)(Result+i*PtrSize),
                            SBytePtr);
@@ -124,9 +126,9 @@ int ExecutionEngine::runFunctionAsMain(Function *Fn,
 
 /// If possible, create a JIT, unless the caller specifically requests an
 /// Interpreter or there's an error. If even an Interpreter cannot be created,
-/// NULL is returned. 
+/// NULL is returned.
 ///
-ExecutionEngine *ExecutionEngine::create(ModuleProvider *MP, 
+ExecutionEngine *ExecutionEngine::create(ModuleProvider *MP,
                                          bool ForceInterpreter,
                                          IntrinsicLowering *IL) {
   ExecutionEngine *EE = 0;
@@ -151,10 +153,10 @@ ExecutionEngine *ExecutionEngine::create(ModuleProvider *MP,
     }
   }
 
-  if (EE == 0) 
+  if (EE == 0)
     delete IL;
   else
-    // Make sure we can resolve symbols in the program as well. The zero arg 
+    // Make sure we can resolve symbols in the program as well. The zero arg
     // to the function tells DynamicLibrary to load the program, not a library.
     sys::DynamicLibrary::LoadLibraryPermanently(0);
 
@@ -168,12 +170,13 @@ void *ExecutionEngine::getPointerToGlobal(const GlobalValue *GV) {
   if (Function *F = const_cast<Function*>(dyn_cast<Function>(GV)))
     return getPointerToFunction(F);
 
-  assert(GlobalAddressMap[GV] && "Global hasn't had an address allocated yet?");
-  return GlobalAddressMap[GV];
+  MutexGuard locked(lock);
+  assert(state.getGlobalAddressMap(locked)[GV] && "Global hasn't had an address allocated yet?");
+  return state.getGlobalAddressMap(locked)[GV];
 }
 
 /// FIXME: document
-/// 
+///
 GenericValue ExecutionEngine::getConstantValue(const Constant *C) {
   GenericValue Result;
   if (isa<UndefValue>(C)) return Result;
@@ -185,8 +188,11 @@ GenericValue ExecutionEngine::getConstantValue(const Constant *C) {
       std::vector<Value*> Indexes(CE->op_begin()+1, CE->op_end());
       uint64_t Offset =
         TD->getIndexedOffset(CE->getOperand(0)->getType(), Indexes);
-                             
-      Result.LongVal += Offset;
+
+      if (getTargetData().getPointerSize() == 4)
+        Result.IntVal += Offset;
+      else
+        Result.LongVal += Offset;
       return Result;
     }
     case Instruction::Cast: {
@@ -203,7 +209,7 @@ GenericValue ExecutionEngine::getConstantValue(const Constant *C) {
       // Handle a cast of pointer to any integral type...
       if (isa<PointerType>(Op->getType()) && C->getType()->isIntegral())
         return GV;
-        
+
       // Handle cast of integer to a pointer...
       if (isa<PointerType>(C->getType()) && Op->getType()->isIntegral())
         switch (Op->getType()->getTypeID()) {
@@ -260,21 +266,21 @@ GenericValue ExecutionEngine::getConstantValue(const Constant *C) {
     std::cerr << "ConstantExpr not handled as global var init: " << *CE << "\n";
     abort();
   }
-  
+
   switch (C->getType()->getTypeID()) {
-#define GET_CONST_VAL(TY, CLASS) \
-  case Type::TY##TyID: Result.TY##Val = cast<CLASS>(C)->getValue(); break
-    GET_CONST_VAL(Bool   , ConstantBool);
-    GET_CONST_VAL(UByte  , ConstantUInt);
-    GET_CONST_VAL(SByte  , ConstantSInt);
-    GET_CONST_VAL(UShort , ConstantUInt);
-    GET_CONST_VAL(Short  , ConstantSInt);
-    GET_CONST_VAL(UInt   , ConstantUInt);
-    GET_CONST_VAL(Int    , ConstantSInt);
-    GET_CONST_VAL(ULong  , ConstantUInt);
-    GET_CONST_VAL(Long   , ConstantSInt);
-    GET_CONST_VAL(Float  , ConstantFP);
-    GET_CONST_VAL(Double , ConstantFP);
+#define GET_CONST_VAL(TY, CTY, CLASS) \
+  case Type::TY##TyID: Result.TY##Val = (CTY)cast<CLASS>(C)->getValue(); break
+    GET_CONST_VAL(Bool   , bool          , ConstantBool);
+    GET_CONST_VAL(UByte  , unsigned char , ConstantUInt);
+    GET_CONST_VAL(SByte  , signed char   , ConstantSInt);
+    GET_CONST_VAL(UShort , unsigned short, ConstantUInt);
+    GET_CONST_VAL(Short  , signed short  , ConstantSInt);
+    GET_CONST_VAL(UInt   , unsigned int  , ConstantUInt);
+    GET_CONST_VAL(Int    , signed int    , ConstantSInt);
+    GET_CONST_VAL(ULong  , uint64_t      , ConstantUInt);
+    GET_CONST_VAL(Long   , int64_t       , ConstantSInt);
+    GET_CONST_VAL(Float  , float         , ConstantFP);
+    GET_CONST_VAL(Double , double        , ConstantFP);
 #undef GET_CONST_VAL
   case Type::PointerTyID:
     if (isa<ConstantPointerNull>(C))
@@ -318,15 +324,16 @@ void ExecutionEngine::StoreValueToMemory(GenericValue Val, GenericValue *Ptr,
                               goto Store4BytesLittleEndian;
     case Type::DoubleTyID:
     case Type::ULongTyID:
-    case Type::LongTyID:    Ptr->Untyped[0] =  Val.ULongVal        & 255;
-                            Ptr->Untyped[1] = (Val.ULongVal >>  8) & 255;
-                            Ptr->Untyped[2] = (Val.ULongVal >> 16) & 255;
-                            Ptr->Untyped[3] = (Val.ULongVal >> 24) & 255;
-                            Ptr->Untyped[4] = (Val.ULongVal >> 32) & 255;
-                            Ptr->Untyped[5] = (Val.ULongVal >> 40) & 255;
-                            Ptr->Untyped[6] = (Val.ULongVal >> 48) & 255;
-                            Ptr->Untyped[7] = (Val.ULongVal >> 56) & 255;
-                            break;
+    case Type::LongTyID:
+      Ptr->Untyped[0] = (unsigned char)(Val.ULongVal      );
+      Ptr->Untyped[1] = (unsigned char)(Val.ULongVal >>  8);
+      Ptr->Untyped[2] = (unsigned char)(Val.ULongVal >> 16);
+      Ptr->Untyped[3] = (unsigned char)(Val.ULongVal >> 24);
+      Ptr->Untyped[4] = (unsigned char)(Val.ULongVal >> 32);
+      Ptr->Untyped[5] = (unsigned char)(Val.ULongVal >> 40);
+      Ptr->Untyped[6] = (unsigned char)(Val.ULongVal >> 48);
+      Ptr->Untyped[7] = (unsigned char)(Val.ULongVal >> 56);
+      break;
     default:
       std::cout << "Cannot store value of type " << *Ty << "!\n";
     }
@@ -351,15 +358,16 @@ void ExecutionEngine::StoreValueToMemory(GenericValue Val, GenericValue *Ptr,
                               goto Store4BytesBigEndian;
     case Type::DoubleTyID:
     case Type::ULongTyID:
-    case Type::LongTyID:    Ptr->Untyped[7] =  Val.ULongVal        & 255;
-                            Ptr->Untyped[6] = (Val.ULongVal >>  8) & 255;
-                            Ptr->Untyped[5] = (Val.ULongVal >> 16) & 255;
-                            Ptr->Untyped[4] = (Val.ULongVal >> 24) & 255;
-                            Ptr->Untyped[3] = (Val.ULongVal >> 32) & 255;
-                            Ptr->Untyped[2] = (Val.ULongVal >> 40) & 255;
-                            Ptr->Untyped[1] = (Val.ULongVal >> 48) & 255;
-                            Ptr->Untyped[0] = (Val.ULongVal >> 56) & 255;
-                            break;
+    case Type::LongTyID:
+      Ptr->Untyped[7] = (unsigned char)(Val.ULongVal      );
+      Ptr->Untyped[6] = (unsigned char)(Val.ULongVal >>  8);
+      Ptr->Untyped[5] = (unsigned char)(Val.ULongVal >> 16);
+      Ptr->Untyped[4] = (unsigned char)(Val.ULongVal >> 24);
+      Ptr->Untyped[3] = (unsigned char)(Val.ULongVal >> 32);
+      Ptr->Untyped[2] = (unsigned char)(Val.ULongVal >> 40);
+      Ptr->Untyped[1] = (unsigned char)(Val.ULongVal >> 48);
+      Ptr->Untyped[0] = (unsigned char)(Val.ULongVal >> 56);
+      break;
     default:
       std::cout << "Cannot store value of type " << *Ty << "!\n";
     }
@@ -380,7 +388,7 @@ GenericValue ExecutionEngine::LoadValueFromMemory(GenericValue *Ptr,
     case Type::ShortTyID:   Result.UShortVal = (unsigned)Ptr->Untyped[0] |
                                               ((unsigned)Ptr->Untyped[1] << 8);
                             break;
-    Load4BytesLittleEndian:                            
+    Load4BytesLittleEndian:
     case Type::FloatTyID:
     case Type::UIntTyID:
     case Type::IntTyID:     Result.UIntVal = (unsigned)Ptr->Untyped[0] |
@@ -448,25 +456,21 @@ GenericValue ExecutionEngine::LoadValueFromMemory(GenericValue *Ptr,
 //
 void ExecutionEngine::InitializeMemory(const Constant *Init, void *Addr) {
   if (isa<UndefValue>(Init)) {
-    // FIXME: THIS SHOULD NOT BE NEEDED.
-    unsigned Size = getTargetData().getTypeSize(Init->getType());
-    memset(Addr, 0, Size);
     return;
   } else if (Init->getType()->isFirstClassType()) {
     GenericValue Val = getConstantValue(Init);
     StoreValueToMemory(Val, (GenericValue*)Addr, Init->getType());
     return;
   } else if (isa<ConstantAggregateZero>(Init)) {
-    unsigned Size = getTargetData().getTypeSize(Init->getType());
-    memset(Addr, 0, Size);
+    memset(Addr, 0, (size_t)getTargetData().getTypeSize(Init->getType()));
     return;
   }
 
   switch (Init->getType()->getTypeID()) {
   case Type::ArrayTyID: {
     const ConstantArray *CPA = cast<ConstantArray>(Init);
-    unsigned ElementSize = 
-      getTargetData().getTypeSize(cast<ArrayType>(CPA->getType())->getElementType());
+    unsigned ElementSize =
+      getTargetData().getTypeSize(CPA->getType()->getElementType());
     for (unsigned i = 0, e = CPA->getNumOperands(); i != e; ++i)
       InitializeMemory(CPA->getOperand(i), (char*)Addr+i*ElementSize);
     return;
@@ -493,15 +497,16 @@ void ExecutionEngine::InitializeMemory(const Constant *Init, void *Addr) {
 ///
 void ExecutionEngine::emitGlobals() {
   const TargetData &TD = getTargetData();
-  
+
   // Loop over all of the global variables in the program, allocating the memory
   // to hold them.
-  for (Module::giterator I = getModule().gbegin(), E = getModule().gend();
+  Module &M = getModule();
+  for (Module::const_global_iterator I = M.global_begin(), E = M.global_end();
        I != E; ++I)
     if (!I->isExternal()) {
       // Get the type of the global...
       const Type *Ty = I->getType()->getElementType();
-      
+
       // Allocate some memory for it!
       unsigned Size = TD.getTypeSize(Ty);
       addGlobalMapping(I, new char[Size]);
@@ -517,10 +522,10 @@ void ExecutionEngine::emitGlobals() {
         abort();
       }
     }
-  
+
   // Now that all of the globals are set up in memory, loop through them all and
   // initialize their contents.
-  for (Module::giterator I = getModule().gbegin(), E = getModule().gend();
+  for (Module::const_global_iterator I = M.global_begin(), E = M.global_end();
        I != E; ++I)
     if (!I->isExternal())
       EmitGlobalVariable(I);
@@ -534,7 +539,7 @@ void ExecutionEngine::EmitGlobalVariable(const GlobalVariable *GV) {
   DEBUG(std::cerr << "Global '" << GV->getName() << "' -> " << GA << "\n");
 
   const Type *ElTy = GV->getType()->getElementType();
-  unsigned GVSize = getTargetData().getTypeSize(ElTy);
+  size_t GVSize = (size_t)getTargetData().getTypeSize(ElTy);
   if (GA == 0) {
     // If it's not already specified, allocate memory for the global.
     GA = new char[GVSize];
@@ -542,6 +547,6 @@ void ExecutionEngine::EmitGlobalVariable(const GlobalVariable *GV) {
   }
 
   InitializeMemory(GV->getInitializer(), GA);
-  NumInitBytes += GVSize;
+  NumInitBytes += (unsigned)GVSize;
   ++NumGlobals;
 }
