@@ -1,16 +1,16 @@
 //===-- ExternalFunctions.cpp - Implement External Functions --------------===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
-// 
+//
 //  This file contains both code to deal with invoking "external" functions, but
 //  also contains code that implements "exported" external functions.
 //
-//  External functions in the interpreter are implemented by 
+//  External functions in the interpreter are implemented by
 //  using the system's dynamic loader to look up the address of the function
 //  we want to invoke.  If a function is found, then one of the
 //  many lle_* wrapper functions in this file will translate its arguments from
@@ -82,22 +82,24 @@ static ExFunc lookupFunction(const Function *F) {
   return FnPtr;
 }
 
-GenericValue Interpreter::callExternalFunction(Function *M,
+GenericValue Interpreter::callExternalFunction(Function *F,
                                      const std::vector<GenericValue> &ArgVals) {
   TheInterpreter = this;
 
   // Do a lookup to see if the function is in our cache... this should just be a
   // deferred annotation!
-  std::map<const Function *, ExFunc>::iterator FI = Functions.find(M);
-  ExFunc Fn = (FI == Functions.end()) ? lookupFunction(M) : FI->second;
+  std::map<const Function *, ExFunc>::iterator FI = Functions.find(F);
+  ExFunc Fn = (FI == Functions.end()) ? lookupFunction(F) : FI->second;
   if (Fn == 0) {
     std::cout << "Tried to execute an unknown external function: "
-              << M->getType()->getDescription() << " " << M->getName() << "\n";
-    return GenericValue();
+              << F->getType()->getDescription() << " " << F->getName() << "\n";
+    if (F->getName() == "__main")
+      return GenericValue();
+    abort();
   }
 
   // TODO: FIXME when types are not const!
-  GenericValue Result = Fn(const_cast<FunctionType*>(M->getFunctionType()),
+  GenericValue Result = Fn(const_cast<FunctionType*>(F->getFunctionType()),
                            ArgVals);
   return Result;
 }
@@ -299,7 +301,7 @@ GenericValue lle_X_sprintf(FunctionType *M, const vector<GenericValue> &Args) {
         Last = *FB++ = *FmtStr++;
       }
       *FB = 0;
-      
+
       switch (Last) {
       case '%':
         sprintf(Buffer, FmtBuf); break;
@@ -326,7 +328,7 @@ GenericValue lle_X_sprintf(FunctionType *M, const vector<GenericValue> &Args) {
         sprintf(Buffer, FmtBuf, Args[ArgNo++].DoubleVal); break;
       case 'p':
         sprintf(Buffer, FmtBuf, (void*)GVTOP(Args[ArgNo++])); break;
-      case 's': 
+      case 's':
         sprintf(Buffer, FmtBuf, (char*)GVTOP(Args[ArgNo++])); break;
       default:  std::cout << "<unknown printf code '" << *FmtStr << "'!>";
         ArgNo++; break;
@@ -461,7 +463,7 @@ GenericValue lle_X_scanf(FunctionType *M, const vector<GenericValue> &args) {
 
 // int clock(void) - Profiling implementation
 GenericValue lle_i_clock(FunctionType *M, const vector<GenericValue> &Args) {
-  extern int clock(void);
+  extern unsigned int clock(void);
   GenericValue GV; GV.IntVal = clock();
   return GV;
 }
@@ -502,13 +504,13 @@ static GenericValue size_t_to_GV (size_t n) {
   return Ret;
 }
 
-static size_t GV_to_size_t (GenericValue GV) { 
+static size_t GV_to_size_t (GenericValue GV) {
   size_t count;
   if (sizeof (size_t) == sizeof (uint64_t)) {
-    count = GV.ULongVal;
+    count = (size_t)GV.ULongVal;
   } else {
     assert (sizeof (size_t) == sizeof (unsigned int));
-    count = GV.UIntVal;
+    count = (size_t)GV.UIntVal;
   }
   return count;
 }
@@ -558,7 +560,7 @@ GenericValue lle_X_memcpy(FunctionType *M, const vector<GenericValue> &Args) {
 GenericValue lle_X_fopen(FunctionType *M, const vector<GenericValue> &Args) {
   assert(Args.size() == 2);
   return PTOGV(fopen((const char *)GVTOP(Args[0]),
-		     (const char *)GVTOP(Args[1])));
+                     (const char *)GVTOP(Args[1])));
 }
 
 // int fclose(FILE *F);
@@ -602,14 +604,14 @@ GenericValue lle_X_fwrite(FunctionType *M, const vector<GenericValue> &Args) {
 GenericValue lle_X_fgets(FunctionType *M, const vector<GenericValue> &Args) {
   assert(Args.size() == 3);
   return GVTOP(fgets((char*)GVTOP(Args[0]), Args[1].IntVal,
-		     getFILE(GVTOP(Args[2]))));
+                     getFILE(GVTOP(Args[2]))));
 }
 
 // FILE *freopen(const char *path, const char *mode, FILE *stream);
 GenericValue lle_X_freopen(FunctionType *M, const vector<GenericValue> &Args) {
   assert(Args.size() == 3);
   return PTOGV(freopen((char*)GVTOP(Args[0]), (char*)GVTOP(Args[1]),
-		       getFILE(GVTOP(Args[2]))));
+                       getFILE(GVTOP(Args[2]))));
 }
 
 // int fflush(FILE *stream);
