@@ -1,10 +1,10 @@
 //===-- IsInf.cpp - Platform-independent wrapper around C99 isinf() -------===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 
 #include "llvm/Config/config.h"
@@ -26,9 +26,13 @@ static int isinf(double x) { return !finite(x) && x==x; }
 #define isinf(X) (!_finite(X))
 #elif defined(_AIX) && defined(__GNUC__)
 // GCC's fixincludes seems to be removing the isinf() declaration from the
-// system header /usr/include/math.h 
+// system header /usr/include/math.h
 # include <math.h>
 static int isinf(double x) { return !finite(x) && x==x; }
+#elif defined(__hpux)
+// HP-UX is "special"
+#include <math.h>
+static int isinf(double x) { return ((x)==INFINITY)||((x)==-INFINITY); }
 #else
 # error "Don't know how to get isinf()"
 #endif
