@@ -1,10 +1,10 @@
 //===- Memory.cpp - Memory Handling Support ---------------------*- C++ -*-===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
-// This file was developed by Reid Spencer and is distributed under the 
+// This file was developed by Reid Spencer and is distributed under the
 // University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 //
 // This file defines some helpful functions for allocating memory and dealing
@@ -13,18 +13,23 @@
 //===----------------------------------------------------------------------===//
 
 #include "llvm/System/Memory.h"
+#include "llvm/Config/config.h"
 
 namespace llvm {
 using namespace sys;
 
 //===----------------------------------------------------------------------===//
 //=== WARNING: Implementation here must contain only TRULY operating system
-//===          independent code. 
+//===          independent code.
 //===----------------------------------------------------------------------===//
 
 }
 
 // Include the platform-specific parts of this class.
-#include "platform/Memory.cpp"
+#ifdef LLVM_ON_UNIX
+#include "Unix/Memory.inc"
+#endif
+#ifdef LLVM_ON_WIN32
+#include "Win32/Memory.inc"
+#endif
 
-// vim: sw=2 smartindent smarttab tw=80 autoindent expandtab
