@@ -1,21 +1,24 @@
 //===- Win32/Win32.h - Common Win32 Include File ----------------*- C++ -*-===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
-// This file was developed by Reid Spencer and is distributed under the 
+// This file was developed by Jeff Cohen and is distributed under the
 // University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 //
-// This file defines things specific to Unix implementations.
+// This file defines things specific to Win32 implementations.
 //
 //===----------------------------------------------------------------------===//
 
 //===----------------------------------------------------------------------===//
-//=== WARNING: Implementation here must contain only generic UNIX code that
-//===          is guaranteed to work on all UNIX variants.
+//=== WARNING: Implementation here must contain only generic Win32 code that
+//===          is guaranteed to work on *all* Win32 variants.
 //===----------------------------------------------------------------------===//
 
+// Require at least Windows 2000 API.
+#define _WIN32_WINNT 0x0500
+
 #include "llvm/Config/config.h"     // Get autoconf configuration settings
 #include "windows.h"
 #include <cassert>
