@@ -1,12 +1,12 @@
 //===-- CTargetMachine.h - TargetMachine for the C backend ------*- C++ -*-===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
-// 
+//
 // This file declares the TargetMachine that is used by the C backend.
 //
 //===----------------------------------------------------------------------===//
@@ -20,11 +20,13 @@ namespace llvm {
 class IntrinsicLowering;
 
 struct CTargetMachine : public TargetMachine {
-  CTargetMachine(const Module &M, IntrinsicLowering *IL) :
+  CTargetMachine(const Module &M, IntrinsicLowering *IL,
+                 const std::string &FS) :
     TargetMachine("CBackend", IL, M) {}
 
   // This is the only thing that actually does anything here.
-  virtual bool addPassesToEmitAssembly(PassManager &PM, std::ostream &Out);
+  virtual bool addPassesToEmitFile(PassManager &PM, std::ostream &Out,
+                                   CodeGenFileType FileType);
 
   // This class always works, but shouldn't be the default in most cases.
   static unsigned getModuleMatchQuality(const Module &M) { return 1; }
