@@ -44,44 +44,103 @@ namespace {
 X86RegisterInfo::X86RegisterInfo()
   : X86GenRegisterInfo(X86::ADJCALLSTACKDOWN, X86::ADJCALLSTACKUP) {}
 
-static unsigned getIdx(unsigned SpillSize) {
-  switch (SpillSize) {
-  default: assert(0 && "Invalid data size!");
-  case 8:  return 0;
-  case 16: return 1;
-  case 32: return 2;
-  case 64: return 3;   // FP in 64-bit spill mode.
-  case 80: return 4;   // FP in 80-bit spill mode.
-  }
-}
-
 void X86RegisterInfo::storeRegToStackSlot(MachineBasicBlock &MBB,
                                           MachineBasicBlock::iterator MI,
-                                          unsigned SrcReg, int FrameIdx) const {
-  static const unsigned Opcode[] =
-    { X86::MOV8mr, X86::MOV16mr, X86::MOV32mr, X86::FST64m, X86::FSTP80m };
-  unsigned Idx = getIdx(getSpillSize(SrcReg));
-  addFrameReference(BuildMI(MBB, MI, Opcode[Idx], 5), FrameIdx).addReg(SrcReg);
+                                          unsigned SrcReg, int FrameIdx,
+                                          const TargetRegisterClass *RC) const {
+  unsigned Opc;
+  if (RC == &X86::R32RegClass) {
+    Opc = X86::MOV32mr;
+  } else if (RC == &X86::R8RegClass) {
+    Opc = X86::MOV8mr;
+  } else if (RC == &X86::R16RegClass) {
+    Opc = X86::MOV16mr;
+  } else if (RC == &X86::RFPRegClass || RC == &X86::RSTRegClass) {
+    Opc = X86::FST64m;
+  } else if (RC == &X86::V4F4RegClass) {
+    Opc = X86::MOVSSmr;
+  } else if (RC == &X86::V2F8RegClass) {
+    Opc = X86::MOVSDmr;
+  } else {
+    assert(0 && "Unknown regclass");
+    abort();
+  }
+  addFrameReference(BuildMI(MBB, MI, Opc, 5), FrameIdx).addReg(SrcReg);
 }
 
 void X86RegisterInfo::loadRegFromStackSlot(MachineBasicBlock &MBB,
                                            MachineBasicBlock::iterator MI,
-                                           unsigned DestReg, int FrameIdx)const{
-  static const unsigned Opcode[] =
-    { X86::MOV8rm, X86::MOV16rm, X86::MOV32rm, X86::FLD64m, X86::FLD80m };
-  unsigned Idx = getIdx(getSpillSize(DestReg));
-  addFrameReference(BuildMI(MBB, MI, Opcode[Idx], 4, DestReg), FrameIdx);
+                                           unsigned DestReg, int FrameIdx,
+                                           const TargetRegisterClass *RC) const{
+  unsigned Opc;
+  if (RC == &X86::R32RegClass) {
+    Opc = X86::MOV32rm;
+  } else if (RC == &X86::R8RegClass) {
+    Opc = X86::MOV8rm;
+  } else if (RC == &X86::R16RegClass) {
+    Opc = X86::MOV16rm;
+  } else if (RC == &X86::RFPRegClass || RC == &X86::RSTRegClass) {
+    Opc = X86::FLD64m;
+  } else if (RC == &X86::V4F4RegClass) {
+    Opc = X86::MOVSSrm;
+  } else if (RC == &X86::V2F8RegClass) {
+    Opc = X86::MOVSDrm;
+  } else {
+    assert(0 && "Unknown regclass");
+    abort();
+  }
+  addFrameReference(BuildMI(MBB, MI, Opc, 4, DestReg), FrameIdx);
 }
 
 void X86RegisterInfo::copyRegToReg(MachineBasicBlock &MBB,
                                    MachineBasicBlock::iterator MI,
                                    unsigned DestReg, unsigned SrcReg,
                                    const TargetRegisterClass *RC) const {
-  static const unsigned Opcode[] =
-    { X86::MOV8rr, X86::MOV16rr, X86::MOV32rr, X86::FpMOV, X86::FpMOV };
-  BuildMI(MBB, MI, Opcode[getIdx(RC->getSize()*8)], 1, DestReg).addReg(SrcReg);
+  unsigned Opc;
+  if (RC == &X86::R32RegClass) {
+    Opc = X86::MOV32rr;
+  } else if (RC == &X86::R8RegClass) {
+    Opc = X86::MOV8rr;
+  } else if (RC == &X86::R16RegClass) {
+    Opc = X86::MOV16rr;
+  } else if (RC == &X86::RFPRegClass || RC == &X86::RSTRegClass) {
+    Opc = X86::FpMOV;
+  } else if (RC == &X86::V4F4RegClass) {
+    Opc = X86::MOVSSrr;
+  } else if (RC == &X86::V2F8RegClass) {
+    Opc = X86::MOVSDrr;
+  } else {
+    assert(0 && "Unknown regclass");
+    abort();
+  }
+  BuildMI(MBB, MI, Opc, 1, DestReg).addReg(SrcReg);
 }
 
+unsigned X86RegisterInfo::isLoadFromStackSlot(MachineInstr *MI, 
+                                              int &FrameIndex) const {
+  switch (MI->getOpcode()) {
+  default: break;
+  case X86::MOV8rm:
+  case X86::MOV16rm:
+  case X86::MOV32rm:
+  case X86::FLD64m:
+  case X86::FLD80m:
+  case X86::MOVSSrm:
+  case X86::MOVSDrm:
+    if (MI->getOperand(1).isFrameIndex() && MI->getOperand(2).isImmediate() &&
+        MI->getOperand(3).isRegister() && MI->getOperand(4).isImmediate() &&
+        MI->getOperand(2).getImmedValue() == 1 &&
+        MI->getOperand(3).getReg() == 0 &&
+        MI->getOperand(4).getImmedValue() == 0) {
+      FrameIndex = MI->getOperand(1).getFrameIndex();
+      return MI->getOperand(0).getReg();
+    }
+    break;
+  }
+  return 0;
+}
+
+
 static MachineInstr *MakeMInst(unsigned Opcode, unsigned FrameIndex,
                                MachineInstr *MI) {
   return addFrameReference(BuildMI(Opcode, 4), FrameIndex);
@@ -149,6 +208,9 @@ MachineInstr* X86RegisterInfo::foldMemoryOperand(MachineInstr* MI,
     case X86::MUL8r:     return MakeMInst( X86::MUL8m ,  FrameIndex, MI);
     case X86::MUL16r:    return MakeMInst( X86::MUL16m,  FrameIndex, MI);
     case X86::MUL32r:    return MakeMInst( X86::MUL32m,  FrameIndex, MI);
+    case X86::IMUL8r:    return MakeMInst( X86::IMUL8m , FrameIndex, MI);
+    case X86::IMUL16r:   return MakeMInst( X86::IMUL16m, FrameIndex, MI);
+    case X86::IMUL32r:   return MakeMInst( X86::IMUL32m, FrameIndex, MI);
     case X86::DIV8r:     return MakeMInst( X86::DIV8m ,  FrameIndex, MI);
     case X86::DIV16r:    return MakeMInst( X86::DIV16m,  FrameIndex, MI);
     case X86::DIV32r:    return MakeMInst( X86::DIV32m,  FrameIndex, MI);
@@ -221,10 +283,26 @@ MachineInstr* X86RegisterInfo::foldMemoryOperand(MachineInstr* MI,
     case X86::SAR8ri:    return MakeMIInst(X86::SAR8mi , FrameIndex, MI);
     case X86::SAR16ri:   return MakeMIInst(X86::SAR16mi, FrameIndex, MI);
     case X86::SAR32ri:   return MakeMIInst(X86::SAR32mi, FrameIndex, MI);
+    case X86::ROL8rCL:   return MakeMInst( X86::ROL8mCL ,FrameIndex, MI);
+    case X86::ROL16rCL:  return MakeMInst( X86::ROL16mCL,FrameIndex, MI);
+    case X86::ROL32rCL:  return MakeMInst( X86::ROL32mCL,FrameIndex, MI);
+    case X86::ROL8ri:    return MakeMIInst(X86::ROL8mi , FrameIndex, MI);
+    case X86::ROL16ri:   return MakeMIInst(X86::ROL16mi, FrameIndex, MI);
+    case X86::ROL32ri:   return MakeMIInst(X86::ROL32mi, FrameIndex, MI);
+    case X86::ROR8rCL:   return MakeMInst( X86::ROR8mCL ,FrameIndex, MI);
+    case X86::ROR16rCL:  return MakeMInst( X86::ROR16mCL,FrameIndex, MI);
+    case X86::ROR32rCL:  return MakeMInst( X86::ROR32mCL,FrameIndex, MI);
+    case X86::ROR8ri:    return MakeMIInst(X86::ROR8mi , FrameIndex, MI);
+    case X86::ROR16ri:   return MakeMIInst(X86::ROR16mi, FrameIndex, MI);
+    case X86::ROR32ri:   return MakeMIInst(X86::ROR32mi, FrameIndex, MI);
     case X86::SHLD32rrCL:return MakeMRInst( X86::SHLD32mrCL,FrameIndex, MI);
     case X86::SHLD32rri8:return MakeMRIInst(X86::SHLD32mri8,FrameIndex, MI);
     case X86::SHRD32rrCL:return MakeMRInst( X86::SHRD32mrCL,FrameIndex, MI);
     case X86::SHRD32rri8:return MakeMRIInst(X86::SHRD32mri8,FrameIndex, MI);
+    case X86::SHLD16rrCL:return MakeMRInst( X86::SHLD16mrCL,FrameIndex, MI);
+    case X86::SHLD16rri8:return MakeMRIInst(X86::SHLD16mri8,FrameIndex, MI);
+    case X86::SHRD16rrCL:return MakeMRInst( X86::SHRD16mrCL,FrameIndex, MI);
+    case X86::SHRD16rri8:return MakeMRIInst(X86::SHRD16mri8,FrameIndex, MI);
     case X86::SETBr:     return MakeMInst( X86::SETBm,   FrameIndex, MI);
     case X86::SETAEr:    return MakeMInst( X86::SETAEm,  FrameIndex, MI);
     case X86::SETEr:     return MakeMInst( X86::SETEm,   FrameIndex, MI);
@@ -234,6 +312,7 @@ MachineInstr* X86RegisterInfo::foldMemoryOperand(MachineInstr* MI,
     case X86::SETSr:     return MakeMInst( X86::SETSm,   FrameIndex, MI);
     case X86::SETNSr:    return MakeMInst( X86::SETNSm,  FrameIndex, MI);
     case X86::SETPr:     return MakeMInst( X86::SETPm,   FrameIndex, MI);
+    case X86::SETNPr:    return MakeMInst( X86::SETNPm,  FrameIndex, MI);
     case X86::SETLr:     return MakeMInst( X86::SETLm,   FrameIndex, MI);
     case X86::SETGEr:    return MakeMInst( X86::SETGEm,  FrameIndex, MI);
     case X86::SETLEr:    return MakeMInst( X86::SETLEm,  FrameIndex, MI);
@@ -275,6 +354,10 @@ MachineInstr* X86RegisterInfo::foldMemoryOperand(MachineInstr* MI,
     case X86::CMOVS32rr: return MakeRMInst(X86::CMOVS32rm , FrameIndex, MI);
     case X86::CMOVNS16rr: return MakeRMInst(X86::CMOVNS16rm , FrameIndex, MI);
     case X86::CMOVNS32rr: return MakeRMInst(X86::CMOVNS32rm , FrameIndex, MI);
+    case X86::CMOVP16rr: return MakeRMInst(X86::CMOVP16rm , FrameIndex, MI);
+    case X86::CMOVP32rr: return MakeRMInst(X86::CMOVP32rm , FrameIndex, MI);
+    case X86::CMOVNP16rr: return MakeRMInst(X86::CMOVNP16rm , FrameIndex, MI);
+    case X86::CMOVNP32rr: return MakeRMInst(X86::CMOVNP32rm , FrameIndex, MI);
     case X86::CMOVL16rr: return MakeRMInst(X86::CMOVL16rm , FrameIndex, MI);
     case X86::CMOVL32rr: return MakeRMInst(X86::CMOVL32rm , FrameIndex, MI);
     case X86::CMOVGE16rr: return MakeRMInst(X86::CMOVGE16rm , FrameIndex, MI);
@@ -351,17 +434,31 @@ eliminateCallFramePseudoInstr(MachineFunction &MF, MachineBasicBlock &MBB,
       unsigned Align = MF.getTarget().getFrameInfo()->getStackAlignment();
       Amount = (Amount+Align-1)/Align*Align;
 
-      MachineInstr *New;
+      MachineInstr *New = 0;
       if (Old->getOpcode() == X86::ADJCALLSTACKDOWN) {
-	New=BuildMI(X86::SUB32ri, 1, X86::ESP, MachineOperand::UseAndDef)
+        New=BuildMI(X86::SUB32ri, 1, X86::ESP, MachineOperand::UseAndDef)
               .addZImm(Amount);
       } else {
-	assert(Old->getOpcode() == X86::ADJCALLSTACKUP);
-	New=BuildMI(X86::ADD32ri, 1, X86::ESP, MachineOperand::UseAndDef)
-              .addZImm(Amount);
+        assert(Old->getOpcode() == X86::ADJCALLSTACKUP);
+        // factor out the amount the callee already popped.
+        unsigned CalleeAmt = Old->getOperand(1).getImmedValue();
+        Amount -= CalleeAmt;
+        if (Amount)
+          New = BuildMI(X86::ADD32ri, 1, X86::ESP,
+                        MachineOperand::UseAndDef).addZImm(Amount);
       }
 
       // Replace the pseudo instruction with a new instruction...
+      if (New) MBB.insert(I, New);
+    }
+  } else if (I->getOpcode() == X86::ADJCALLSTACKUP) {
+    // If we are performing frame pointer elimination and if the callee pops
+    // something off the stack pointer, add it back.  We do this until we have
+    // more advanced stack pointer tracking ability.
+    if (unsigned CalleeAmt = I->getOperand(1).getImmedValue()) {
+      MachineInstr *New =
+        BuildMI(X86::SUB32ri, 1, X86::ESP,
+                MachineOperand::UseAndDef).addZImm(CalleeAmt);
       MBB.insert(I, New);
     }
   }
@@ -427,7 +524,7 @@ void X86RegisterInfo::emitPrologue(MachineFunction &MF) const {
 
     // Save EBP into the appropriate stack slot...
     MI = addRegOffset(BuildMI(X86::MOV32mr, 5),    // mov [ESP-<offset>], EBP
-		      X86::ESP, EBPOffset+NumBytes).addReg(X86::EBP);
+                      X86::ESP, EBPOffset+NumBytes).addReg(X86::EBP);
     MBB.insert(MBBI, MI);
 
     // Update EBP with the new base value...
@@ -468,9 +565,16 @@ void X86RegisterInfo::emitEpilogue(MachineFunction &MF,
                                    MachineBasicBlock &MBB) const {
   const MachineFrameInfo *MFI = MF.getFrameInfo();
   MachineBasicBlock::iterator MBBI = prior(MBB.end());
-  MachineInstr *MI;
-  assert(MBBI->getOpcode() == X86::RET &&
-         "Can only insert epilog into returning blocks");
+
+  switch (MBBI->getOpcode()) {
+  case X86::RET:
+  case X86::RETI:
+  case X86::TAILJMPd:
+  case X86::TAILJMPr:
+  case X86::TAILJMPm: break;  // These are ok
+  default:
+    assert(0 && "Can only insert epilog into returning blocks");
+  }
 
   if (hasFP(MF)) {
     // Get the offset of the stack slot for the EBP register... which is
@@ -478,42 +582,42 @@ void X86RegisterInfo::emitEpilogue(MachineFunction &MF,
     int EBPOffset = MFI->getObjectOffset(MFI->getObjectIndexEnd()-1)+4;
 
     // mov ESP, EBP
-    MI = BuildMI(X86::MOV32rr, 1,X86::ESP).addReg(X86::EBP);
-    MBB.insert(MBBI, MI);
+    BuildMI(MBB, MBBI, X86::MOV32rr, 1,X86::ESP).addReg(X86::EBP);
 
     // pop EBP
-    MI = BuildMI(X86::POP32r, 0, X86::EBP);
-    MBB.insert(MBBI, MI);
+    BuildMI(MBB, MBBI, X86::POP32r, 0, X86::EBP);
   } else {
     // Get the number of bytes allocated from the FrameInfo...
     unsigned NumBytes = MFI->getStackSize();
 
     if (NumBytes) {    // adjust stack pointer back: ESP += numbytes
-      MI =BuildMI(X86::ADD32ri, 1, X86::ESP, MachineOperand::UseAndDef)
-            .addZImm(NumBytes);
-      MBB.insert(MBBI, MI);
+      // If there is an ADD32ri or SUB32ri of ESP immediately before this
+      // instruction, merge the two instructions.
+      if (MBBI != MBB.begin()) {
+        MachineBasicBlock::iterator PI = prior(MBBI);
+        if (PI->getOpcode() == X86::ADD32ri &&
+            PI->getOperand(0).getReg() == X86::ESP) {
+          NumBytes += PI->getOperand(1).getImmedValue();
+          MBB.erase(PI);
+        } else if (PI->getOpcode() == X86::SUB32ri &&
+                   PI->getOperand(0).getReg() == X86::ESP) {
+          NumBytes -= PI->getOperand(1).getImmedValue();
+          MBB.erase(PI);
+        } else if (PI->getOpcode() == X86::ADJSTACKPTRri) {
+          NumBytes += PI->getOperand(1).getImmedValue();
+          MBB.erase(PI);
+        }
+      }
+
+      if (NumBytes > 0)
+        BuildMI(MBB, MBBI, X86::ADD32ri, 2)
+          .addReg(X86::ESP, MachineOperand::UseAndDef).addZImm(NumBytes);
+      else if ((int)NumBytes < 0)
+        BuildMI(MBB, MBBI, X86::SUB32ri, 2)
+          .addReg(X86::ESP, MachineOperand::UseAndDef).addZImm(-NumBytes);
     }
   }
 }
 
 #include "X86GenRegisterInfo.inc"
 
-const TargetRegisterClass*
-X86RegisterInfo::getRegClassForType(const Type* Ty) const {
-  switch (Ty->getTypeID()) {
-  case Type::LongTyID:
-  case Type::ULongTyID: assert(0 && "Long values can't fit in registers!");
-  default:              assert(0 && "Invalid type to getClass!");
-  case Type::BoolTyID:
-  case Type::SByteTyID:
-  case Type::UByteTyID:   return &R8Instance;
-  case Type::ShortTyID:
-  case Type::UShortTyID:  return &R16Instance;
-  case Type::IntTyID:
-  case Type::UIntTyID:
-  case Type::PointerTyID: return &R32Instance;
-
-  case Type::FloatTyID:
-  case Type::DoubleTyID: return &RFPInstance;
-  }
-}
