@@ -1,10 +1,10 @@
 //===- IndVarSimplify.cpp - Induction Variable Elimination ----------------===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 //
 // This transformation analyzes and transforms the induction variables (and
@@ -42,7 +42,7 @@
 #include "llvm/Constants.h"
 #include "llvm/Instructions.h"
 #include "llvm/Type.h"
-#include "llvm/Analysis/ScalarEvolutionExpressions.h"
+#include "llvm/Analysis/ScalarEvolutionExpander.h"
 #include "llvm/Analysis/LoopInfo.h"
 #include "llvm/Support/CFG.h"
 #include "llvm/Support/GetElementPtrTypeIterator.h"
@@ -51,241 +51,6 @@
 #include "llvm/ADT/Statistic.h"
 using namespace llvm;
 
-namespace {
-  /// SCEVExpander - This class uses information about analyze scalars to
-  /// rewrite expressions in canonical form.
-  ///
-  /// Clients should create an instance of this class when rewriting is needed,
-  /// and destroying it when finished to allow the release of the associated
-  /// memory.
-  struct SCEVExpander : public SCEVVisitor<SCEVExpander, Value*> {
-    ScalarEvolution &SE;
-    LoopInfo &LI;
-    std::map<SCEVHandle, Value*> InsertedExpressions;
-    std::set<Instruction*> InsertedInstructions;
-
-    Instruction *InsertPt;
-
-    friend struct SCEVVisitor<SCEVExpander, Value*>;
-  public:
-    SCEVExpander(ScalarEvolution &se, LoopInfo &li) : SE(se), LI(li) {}
-
-    /// isInsertedInstruction - Return true if the specified instruction was
-    /// inserted by the code rewriter.  If so, the client should not modify the
-    /// instruction.
-    bool isInsertedInstruction(Instruction *I) const {
-      return InsertedInstructions.count(I);
-    }
-    
-    /// getOrInsertCanonicalInductionVariable - This method returns the
-    /// canonical induction variable of the specified type for the specified
-    /// loop (inserting one if there is none).  A canonical induction variable
-    /// starts at zero and steps by one on each iteration.
-    Value *getOrInsertCanonicalInductionVariable(const Loop *L, const Type *Ty){
-      assert((Ty->isInteger() || Ty->isFloatingPoint()) &&
-             "Can only insert integer or floating point induction variables!");
-      SCEVHandle H = SCEVAddRecExpr::get(SCEVUnknown::getIntegerSCEV(0, Ty),
-                                         SCEVUnknown::getIntegerSCEV(1, Ty), L);
-      return expand(H);
-    }
-
-    /// addInsertedValue - Remember the specified instruction as being the
-    /// canonical form for the specified SCEV.
-    void addInsertedValue(Instruction *I, SCEV *S) {
-      InsertedExpressions[S] = (Value*)I;
-      InsertedInstructions.insert(I);
-    }
-
-    /// expandCodeFor - Insert code to directly compute the specified SCEV
-    /// expression into the program.  The inserted code is inserted into the
-    /// specified block.
-    ///
-    /// If a particular value sign is required, a type may be specified for the
-    /// result.
-    Value *expandCodeFor(SCEVHandle SH, Instruction *IP, const Type *Ty = 0) {
-      // Expand the code for this SCEV.
-      this->InsertPt = IP;
-      return expandInTy(SH, Ty);
-    }
-
-  protected:
-    Value *expand(SCEV *S) {
-      // Check to see if we already expanded this.
-      std::map<SCEVHandle, Value*>::iterator I = InsertedExpressions.find(S);
-      if (I != InsertedExpressions.end())
-        return I->second;
-
-      Value *V = visit(S);
-      InsertedExpressions[S] = V;
-      return V;
-    }
-
-    Value *expandInTy(SCEV *S, const Type *Ty) {
-      Value *V = expand(S);
-      if (Ty && V->getType() != Ty) {
-        // FIXME: keep track of the cast instruction.
-        if (Constant *C = dyn_cast<Constant>(V))
-          return ConstantExpr::getCast(C, Ty);
-        else if (Instruction *I = dyn_cast<Instruction>(V)) {
-          // Check to see if there is already a cast.  If there is, use it.
-          for (Value::use_iterator UI = I->use_begin(), E = I->use_end(); 
-               UI != E; ++UI) {
-            if ((*UI)->getType() == Ty)
-              if (CastInst *CI = dyn_cast<CastInst>(cast<Instruction>(*UI))) {
-                BasicBlock::iterator It = I; ++It;
-                while (isa<PHINode>(It)) ++It;
-                if (It != BasicBlock::iterator(CI)) {
-                  // Splice the cast immediately after the operand in question.
-                  BasicBlock::InstListType &InstList =
-                    I->getParent()->getInstList();
-                  InstList.splice(It, CI->getParent()->getInstList(), CI);
-                }
-                return CI;
-              }
-          }
-          BasicBlock::iterator IP = I; ++IP;
-          if (InvokeInst *II = dyn_cast<InvokeInst>(I))
-            IP = II->getNormalDest()->begin();
-          while (isa<PHINode>(IP)) ++IP;
-          return new CastInst(V, Ty, V->getName(), IP);
-        } else {
-          // FIXME: check to see if there is already a cast!
-          return new CastInst(V, Ty, V->getName(), InsertPt);
-        }
-      }
-      return V;
-    }
-
-    Value *visitConstant(SCEVConstant *S) {
-      return S->getValue();
-    }
-
-    Value *visitTruncateExpr(SCEVTruncateExpr *S) {
-      Value *V = expand(S->getOperand());
-      return new CastInst(V, S->getType(), "tmp.", InsertPt);
-    }
-
-    Value *visitZeroExtendExpr(SCEVZeroExtendExpr *S) {
-      Value *V = expandInTy(S->getOperand(),S->getType()->getUnsignedVersion());
-      return new CastInst(V, S->getType(), "tmp.", InsertPt);
-    }
-
-    Value *visitAddExpr(SCEVAddExpr *S) {
-      const Type *Ty = S->getType();
-      Value *V = expandInTy(S->getOperand(S->getNumOperands()-1), Ty);
-
-      // Emit a bunch of add instructions
-      for (int i = S->getNumOperands()-2; i >= 0; --i)
-        V = BinaryOperator::createAdd(V, expandInTy(S->getOperand(i), Ty),
-                                      "tmp.", InsertPt);
-      return V;
-    }
-
-    Value *visitMulExpr(SCEVMulExpr *S);
-
-    Value *visitUDivExpr(SCEVUDivExpr *S) {
-      const Type *Ty = S->getType();
-      Value *LHS = expandInTy(S->getLHS(), Ty);
-      Value *RHS = expandInTy(S->getRHS(), Ty);
-      return BinaryOperator::createDiv(LHS, RHS, "tmp.", InsertPt);
-    }
-
-    Value *visitAddRecExpr(SCEVAddRecExpr *S);
-
-    Value *visitUnknown(SCEVUnknown *S) {
-      return S->getValue();
-    }
-  };
-}
-
-Value *SCEVExpander::visitMulExpr(SCEVMulExpr *S) {
-  const Type *Ty = S->getType();
-  int FirstOp = 0;  // Set if we should emit a subtract.
-  if (SCEVConstant *SC = dyn_cast<SCEVConstant>(S->getOperand(0)))
-    if (SC->getValue()->isAllOnesValue())
-      FirstOp = 1;
-    
-  int i = S->getNumOperands()-2;
-  Value *V = expandInTy(S->getOperand(i+1), Ty);
-    
-  // Emit a bunch of multiply instructions
-  for (; i >= FirstOp; --i)
-    V = BinaryOperator::createMul(V, expandInTy(S->getOperand(i), Ty),
-                                  "tmp.", InsertPt);
-  // -1 * ...  --->  0 - ...
-  if (FirstOp == 1)
-    V = BinaryOperator::createNeg(V, "tmp.", InsertPt);
-  return V;
-}
-
-Value *SCEVExpander::visitAddRecExpr(SCEVAddRecExpr *S) {
-  const Type *Ty = S->getType();
-  const Loop *L = S->getLoop();
-  // We cannot yet do fp recurrences, e.g. the xform of {X,+,F} --> X+{0,+,F}
-  assert(Ty->isIntegral() && "Cannot expand fp recurrences yet!");
-
-  // {X,+,F} --> X + {0,+,F}
-  if (!isa<SCEVConstant>(S->getStart()) ||
-      !cast<SCEVConstant>(S->getStart())->getValue()->isNullValue()) {
-    Value *Start = expandInTy(S->getStart(), Ty);
-    std::vector<SCEVHandle> NewOps(S->op_begin(), S->op_end());
-    NewOps[0] = SCEVUnknown::getIntegerSCEV(0, Ty);
-    Value *Rest = expandInTy(SCEVAddRecExpr::get(NewOps, L), Ty);
-
-    // FIXME: look for an existing add to use.
-    return BinaryOperator::createAdd(Rest, Start, "tmp.", InsertPt);
-  }
-
-  // {0,+,1} --> Insert a canonical induction variable into the loop!
-  if (S->getNumOperands() == 2 &&
-      S->getOperand(1) == SCEVUnknown::getIntegerSCEV(1, Ty)) {
-    // Create and insert the PHI node for the induction variable in the
-    // specified loop.
-    BasicBlock *Header = L->getHeader();
-    PHINode *PN = new PHINode(Ty, "indvar", Header->begin());
-    PN->addIncoming(Constant::getNullValue(Ty), L->getLoopPreheader());
-
-    pred_iterator HPI = pred_begin(Header);
-    assert(HPI != pred_end(Header) && "Loop with zero preds???");
-    if (!L->contains(*HPI)) ++HPI;
-    assert(HPI != pred_end(Header) && L->contains(*HPI) &&
-           "No backedge in loop?");
-
-    // Insert a unit add instruction right before the terminator corresponding
-    // to the back-edge.
-    Constant *One = Ty->isFloatingPoint() ? (Constant*)ConstantFP::get(Ty, 1.0)
-                                          : ConstantInt::get(Ty, 1);
-    Instruction *Add = BinaryOperator::createAdd(PN, One, "indvar.next",
-                                                 (*HPI)->getTerminator());
-
-    pred_iterator PI = pred_begin(Header);
-    if (*PI == L->getLoopPreheader())
-      ++PI;
-    PN->addIncoming(Add, *PI);
-    return PN;
-  }
-
-  // Get the canonical induction variable I for this loop.
-  Value *I = getOrInsertCanonicalInductionVariable(L, Ty);
-
-  if (S->getNumOperands() == 2) {   // {0,+,F} --> i*F
-    Value *F = expandInTy(S->getOperand(1), Ty);
-    return BinaryOperator::createMul(I, F, "tmp.", InsertPt);
-  }
-
-  // If this is a chain of recurrences, turn it into a closed form, using the
-  // folders, then expandCodeFor the closed form.  This allows the folders to
-  // simplify the expression without having to build a bunch of special code
-  // into this folder.
-  SCEVHandle IH = SCEVUnknown::get(I);   // Get I as a "symbolic" SCEV.
-
-  SCEVHandle V = S->evaluateAtIteration(IH);
-  //std::cerr << "Evaluated: " << *this << "\n     to: " << *V << "\n";
-
-  return expandInTy(V, Ty);
-}
-
-
 namespace {
   Statistic<> NumRemoved ("indvars", "Number of aux indvars removed");
   Statistic<> NumPointer ("indvars", "Number of pointer indvars promoted");
@@ -356,17 +121,17 @@ DeleteTriviallyDeadInstructions(std::set<Instruction*> &Insts) {
 /// EliminatePointerRecurrence - Check to see if this is a trivial GEP pointer
 /// recurrence.  If so, change it into an integer recurrence, permitting
 /// analysis by the SCEV routines.
-void IndVarSimplify::EliminatePointerRecurrence(PHINode *PN, 
+void IndVarSimplify::EliminatePointerRecurrence(PHINode *PN,
                                                 BasicBlock *Preheader,
                                             std::set<Instruction*> &DeadInsts) {
   assert(PN->getNumIncomingValues() == 2 && "Noncanonicalized loop!");
   unsigned PreheaderIdx = PN->getBasicBlockIndex(Preheader);
   unsigned BackedgeIdx = PreheaderIdx^1;
   if (GetElementPtrInst *GEPI =
-      dyn_cast<GetElementPtrInst>(PN->getIncomingValue(BackedgeIdx)))
+          dyn_cast<GetElementPtrInst>(PN->getIncomingValue(BackedgeIdx)))
     if (GEPI->getOperand(0) == PN) {
-      assert(GEPI->getNumOperands() == 2 && "GEP types must mismatch!");
-          
+      assert(GEPI->getNumOperands() == 2 && "GEP types must match!");
+
       // Okay, we found a pointer recurrence.  Transform this pointer
       // recurrence into an integer recurrence.  Compute the value that gets
       // added to the pointer at every iteration.
@@ -381,10 +146,10 @@ void IndVarSimplify::EliminatePointerRecurrence(PHINode *PN,
       Value *NewAdd = BinaryOperator::createAdd(NewPhi, AddedVal,
                                                 GEPI->getName()+".rec", GEPI);
       NewPhi->addIncoming(NewAdd, PN->getIncomingBlock(BackedgeIdx));
-          
+
       // Update the existing GEP to use the recurrence.
       GEPI->setOperand(0, PN->getIncomingValue(PreheaderIdx));
-          
+
       // Update the GEP to use the new recurrence we just inserted.
       GEPI->setOperand(1, NewAdd);
 
@@ -545,11 +310,11 @@ void IndVarSimplify::RewriteLoopExitValues(Loop *L) {
   bool HasConstantItCount = isa<SCEVConstant>(SE->getIterationCount(L));
 
   std::set<Instruction*> InstructionsToDelete;
-  
+
   for (unsigned i = 0, e = L->getBlocks().size(); i != e; ++i)
     if (LI->getLoopFor(L->getBlocks()[i]) == L) {  // Not in a subloop...
       BasicBlock *BB = L->getBlocks()[i];
-      for (BasicBlock::iterator I = BB->begin(), E = BB->end(); I != E; ++I)
+      for (BasicBlock::iterator I = BB->begin(), E = BB->end(); I != E;) {
         if (I->getType()->isInteger()) {      // Is an integer instruction
           SCEVHandle SH = SE->getSCEV(I);
           if (SH->hasComputableLoopEvolution(L) ||    // Varies predictably
@@ -569,6 +334,10 @@ void IndVarSimplify::RewriteLoopExitValues(Loop *L) {
               if (!isa<SCEVCouldNotCompute>(ExitValue)) {
                 Changed = true;
                 ++NumReplaced;
+                // Remember the next instruction.  The rewriter can move code
+                // around in some cases.
+                BasicBlock::iterator NextI = I; ++NextI;
+
                 Value *NewVal = Rewriter.expandCodeFor(ExitValue, InsertPt,
                                                        I->getType());
 
@@ -580,10 +349,16 @@ void IndVarSimplify::RewriteLoopExitValues(Loop *L) {
                 // If this instruction is dead now, schedule it to be removed.
                 if (I->use_empty())
                   InstructionsToDelete.insert(I);
+                I = NextI;
+                continue;  // Skip the ++I
               }
             }
           }
         }
+
+        // Next instruction.  Continue instruction skips this.
+        ++I;
+      }
     }
 
   DeleteTriviallyDeadInstructions(InstructionsToDelete);
@@ -597,7 +372,7 @@ void IndVarSimplify::runOnLoop(Loop *L) {
   //
   BasicBlock *Header    = L->getHeader();
   BasicBlock *Preheader = L->getLoopPreheader();
-  
+
   std::set<Instruction*> DeadInsts;
   for (BasicBlock::iterator I = Header->begin(); isa<PHINode>(I); ++I) {
     PHINode *PN = cast<PHINode>(I);
@@ -632,13 +407,13 @@ void IndVarSimplify::runOnLoop(Loop *L) {
     if (PN->getType()->isInteger()) {  // FIXME: when we have fast-math, enable!
       SCEVHandle SCEV = SE->getSCEV(PN);
       if (SCEV->hasComputableLoopEvolution(L))
-        // FIXME: Without a strength reduction pass, it is an extremely bad idea
-        // to indvar substitute anything more complex than a linear induction
-        // variable.  Doing so will put expensive multiply instructions inside
-        // of the loop.  For now just disable indvar subst on anything more
-        // complex than a linear addrec.
+        // FIXME: It is an extremely bad idea to indvar substitute anything more
+        // complex than affine induction variables.  Doing so will put expensive
+        // polynomial evaluations inside of the loop, and the str reduction pass
+        // currently can only reduce affine polynomials.  For now just disable
+        // indvar subst on anything more complex than an affine addrec.
         if (SCEVAddRecExpr *AR = dyn_cast<SCEVAddRecExpr>(SCEV))
-          if (AR->getNumOperands() == 2 && isa<SCEVConstant>(AR->getOperand(1)))
+          if (AR->isAffine())
             IndVars.push_back(std::make_pair(PN, SCEV));
     }
   }
@@ -746,7 +521,7 @@ void IndVarSimplify::runOnLoop(Loop *L) {
             DeadInsts.insert(I);
             ++NumRemoved;
             Changed = true;
-          }          
+          }
         }
     }
 #endif
