@@ -1,10 +1,10 @@
 //===-- Type.cpp - Implement the Type class -------------------------------===//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 //
 // This file implements the Type class for the VMCore library.
@@ -103,13 +103,13 @@ const Type *Type::getUnsignedVersion() const {
   switch (getTypeID()) {
   default:
     assert(isInteger()&&"Type::getUnsignedVersion is only valid for integers!");
-  case Type::UByteTyID:   
+  case Type::UByteTyID:
   case Type::SByteTyID:   return Type::UByteTy;
-  case Type::UShortTyID:  
+  case Type::UShortTyID:
   case Type::ShortTyID:   return Type::UShortTy;
-  case Type::UIntTyID:    
+  case Type::UIntTyID:
   case Type::IntTyID:     return Type::UIntTy;
-  case Type::ULongTyID:   
+  case Type::ULongTyID:
   case Type::LongTyID:    return Type::ULongTy;
   }
 }
@@ -120,13 +120,13 @@ const Type *Type::getSignedVersion() const {
   switch (getTypeID()) {
   default:
     assert(isInteger() && "Type::getSignedVersion is only valid for integers!");
-  case Type::UByteTyID:   
+  case Type::UByteTyID:
   case Type::SByteTyID:   return Type::SByteTy;
-  case Type::UShortTyID:  
+  case Type::UShortTyID:
   case Type::ShortTyID:   return Type::ShortTy;
-  case Type::UIntTyID:    
+  case Type::UIntTyID:
   case Type::IntTyID:     return Type::IntTy;
-  case Type::ULongTyID:   
+  case Type::ULongTyID:
   case Type::LongTyID:    return Type::LongTy;
   }
 }
@@ -138,8 +138,34 @@ const Type *Type::getSignedVersion() const {
 //
 unsigned Type::getPrimitiveSize() const {
   switch (getTypeID()) {
-#define HANDLE_PRIM_TYPE(TY,SIZE)  case TY##TyID: return SIZE;
-#include "llvm/Type.def"
+  case Type::BoolTyID:
+  case Type::SByteTyID:
+  case Type::UByteTyID: return 1;
+  case Type::UShortTyID:
+  case Type::ShortTyID: return 2;
+  case Type::FloatTyID:
+  case Type::IntTyID:
+  case Type::UIntTyID: return 4;
+  case Type::LongTyID:
+  case Type::ULongTyID:
+  case Type::DoubleTyID: return 8;
+  default: return 0;
+  }
+}
+
+unsigned Type::getPrimitiveSizeInBits() const {
+  switch (getTypeID()) {
+  case Type::BoolTyID:  return 1;
+  case Type::SByteTyID:
+  case Type::UByteTyID: return 8;
+  case Type::UShortTyID:
+  case Type::ShortTyID: return 16;
+  case Type::FloatTyID:
+  case Type::IntTyID:
+  case Type::UIntTyID: return 32;
+  case Type::LongTyID:
+  case Type::ULongTyID:
+  case Type::DoubleTyID: return 64;
   default: return 0;
   }
 }
@@ -167,7 +193,7 @@ bool Type::isSizedDerivedType() const {
 /// algorithm for when a type is being forwarded to another type.
 const Type *Type::getForwardedTypeInternal() const {
   assert(ForwardType && "This type is not being forwarded to another type!");
-  
+
   // Check to see if the forwarded type has been forwarded on.  If so, collapse
   // the forwarding links.
   const Type *RealForwardedType = ForwardType->getForwardedType();
@@ -181,7 +207,7 @@ const Type *Type::getForwardedTypeInternal() const {
 
   // Now drop the old reference.  This could cause ForwardType to get deleted.
   cast<DerivedType>(ForwardType)->dropRef();
-  
+
   // Return the updated type.
   ForwardType = RealForwardedType;
   return ForwardType;
@@ -201,28 +227,28 @@ static std::string getTypeDescription(const Type *Ty,
     AbstractTypeDescriptions.insert(std::make_pair(Ty, Desc));
     return Desc;
   }
-  
+
   if (!Ty->isAbstract()) {                       // Base case for the recursion
     std::map<const Type*, std::string>::iterator I =
       ConcreteTypeDescriptions.find(Ty);
     if (I != ConcreteTypeDescriptions.end()) return I->second;
   }
-      
+
   // Check to see if the Type is already on the stack...
   unsigned Slot = 0, CurSize = TypeStack.size();
   while (Slot < CurSize && TypeStack[Slot] != Ty) ++Slot; // Scan for type
-  
-  // This is another base case for the recursion.  In this case, we know 
+
+  // This is another base case for the recursion.  In this case, we know
   // that we have looped back to a type that we have previously visited.
   // Generate the appropriate upreference to handle this.
-  // 
+  //
   if (Slot < CurSize)
     return "\\" + utostr(CurSize-Slot);         // Here's the upreference
 
   // Recursive case: derived types...
   std::string Result;
   TypeStack.push_back(Ty);    // Add us to the stack..
-      
+
   switch (Ty->getTypeID()) {
   case Type::FunctionTyID: {
     const FunctionType *FTy = cast<FunctionType>(Ty);
@@ -289,9 +315,10 @@ static const std::string &getOrCreateDesc(std::map<const Type*,std::string>&Map,
                                           const Type *Ty) {
   std::map<const Type*, std::string>::iterator I = Map.find(Ty);
   if (I != Map.end()) return I->second;
-    
+
   std::vector<const Type *> TypeStack;
-  return Map[Ty] = getTypeDescription(Ty, TypeStack);
+  std::string Result = getTypeDescription(Ty, TypeStack);
+  return Map[Ty] = Result;
 }
 
 
@@ -365,11 +392,11 @@ Type *Type::LabelTy  = &TheLabelTy;
 //===----------------------------------------------------------------------===//
 
 FunctionType::FunctionType(const Type *Result,
-                           const std::vector<const Type*> &Params, 
-                           bool IsVarArgs) : DerivedType(FunctionTyID), 
+                           const std::vector<const Type*> &Params,
+                           bool IsVarArgs) : DerivedType(FunctionTyID),
                                              isVarArgs(IsVarArgs) {
   assert((Result->isFirstClassType() || Result == Type::VoidTy ||
-         isa<OpaqueType>(Result)) && 
+         isa<OpaqueType>(Result)) &&
          "LLVM functions cannot return aggregates");
   bool isAbstract = Result->isAbstract();
   ContainedTys.reserve(Params.size()+1);
@@ -401,7 +428,7 @@ StructType::StructType(const std::vector<const Type*> &Types)
   setAbstract(isAbstract);
 }
 
-ArrayType::ArrayType(const Type *ElType, unsigned NumEl)
+ArrayType::ArrayType(const Type *ElType, uint64_t NumEl)
   : SequentialType(ArrayTyID, ElType) {
   NumElements = NumEl;
 
@@ -414,7 +441,7 @@ PackedType::PackedType(const Type *ElType, unsigned NumEl)
   NumElements = NumEl;
 
   assert(NumEl > 0 && "NumEl of a PackedType must be greater than 0");
-  assert((ElType->isIntegral() || ElType->isFloatingPoint()) && 
+  assert((ElType->isIntegral() || ElType->isFloatingPoint()) &&
          "Elements of a PackedType must be a primitive type");
 }
 
@@ -438,7 +465,7 @@ void DerivedType::dropAllTypeUses() {
   if (!ContainedTys.empty()) {
     while (ContainedTys.size() > 1)
       ContainedTys.pop_back();
-    
+
     // The type must stay abstract.  To do this, we insert a pointer to a type
     // that will never get resolved, thus will always be abstract.
     static Type *AlwaysOpaqueTy = OpaqueType::get();
@@ -462,15 +489,15 @@ namespace llvm {
   template <> struct GraphTraits<TypePromotionGraph> {
     typedef Type NodeType;
     typedef Type::subtype_iterator ChildIteratorType;
-    
+
     static inline NodeType *getEntryNode(TypePromotionGraph G) { return G.Ty; }
-    static inline ChildIteratorType child_begin(NodeType *N) { 
+    static inline ChildIteratorType child_begin(NodeType *N) {
       if (N->isAbstract())
-        return N->subtype_begin(); 
+        return N->subtype_begin();
       else           // No need to process children of concrete types.
-        return N->subtype_end(); 
+        return N->subtype_end();
     }
-    static inline ChildIteratorType child_end(NodeType *N) { 
+    static inline ChildIteratorType child_end(NodeType *N) {
       return N->subtype_end();
     }
   };
@@ -480,8 +507,6 @@ namespace llvm {
 // PromoteAbstractToConcrete - This is a recursive function that walks a type
 // graph calculating whether or not a type is abstract.
 //
-// This method returns true if the type is found to still be abstract.
-//
 void Type::PromoteAbstractToConcrete() {
   if (!isAbstract()) return;
 
@@ -504,14 +529,21 @@ void Type::PromoteAbstractToConcrete() {
         for (Type::subtype_iterator CI = SCC[i]->subtype_begin(),
                E = SCC[i]->subtype_end(); CI != E; ++CI)
           if ((*CI)->isAbstract())
-            return;               // Not going to be concrete, sorry.
-      
+            // If the child type is in our SCC, it doesn't make the entire SCC
+            // abstract unless there is a non-SCC abstract type.
+            if (std::find(SCC.begin(), SCC.end(), *CI) == SCC.end())
+              return;               // Not going to be concrete, sorry.
+
       // Okay, we just discovered this whole SCC is now concrete, mark it as
       // such!
       for (unsigned i = 0, e = SCC.size(); i != e; ++i) {
         assert(SCC[i]->isAbstract() && "Why are we processing concrete types?");
-        
+
         SCC[i]->setAbstract(false);
+      }
+
+      for (unsigned i = 0, e = SCC.size(); i != e; ++i) {
+        assert(!SCC[i]->isAbstract() && "Concrete type became abstract?");
         // The type just became concrete, notify all users!
         cast<DerivedType>(SCC[i])->notifyUsesThatTypeBecameConcrete();
       }
@@ -600,7 +632,7 @@ static bool AbstractTypeHasCycleThrough(const Type *TargetTy, const Type *CurTy,
   if (!VisitedTypes.insert(CurTy).second)
     return false;  // Already been here.
 
-  for (Type::subtype_iterator I = CurTy->subtype_begin(), 
+  for (Type::subtype_iterator I = CurTy->subtype_begin(),
        E = CurTy->subtype_end(); I != E; ++I)
     if (AbstractTypeHasCycleThrough(TargetTy, *I, VisitedTypes))
       return true;
@@ -614,7 +646,7 @@ static bool ConcreteTypeHasCycleThrough(const Type *TargetTy, const Type *CurTy,
   if (!VisitedTypes.insert(CurTy).second)
     return false;  // Already been here.
 
-  for (Type::subtype_iterator I = CurTy->subtype_begin(), 
+  for (Type::subtype_iterator I = CurTy->subtype_begin(),
        E = CurTy->subtype_end(); I != E; ++I)
     if (ConcreteTypeHasCycleThrough(TargetTy, *I, VisitedTypes))
       return true;
@@ -627,7 +659,7 @@ static bool TypeHasCycleThroughItself(const Type *Ty) {
   std::set<const Type*> VisitedTypes;
 
   if (Ty->isAbstract()) {  // Optimized case for abstract types.
-    for (Type::subtype_iterator I = Ty->subtype_begin(), E = Ty->subtype_end(); 
+    for (Type::subtype_iterator I = Ty->subtype_begin(), E = Ty->subtype_end();
          I != E; ++I)
       if (AbstractTypeHasCycleThrough(Ty, *I, VisitedTypes))
         return true;
@@ -664,7 +696,7 @@ class TypeMap {
 
 private:
   void clear(std::vector<Type *> &DerivedTypes) {
-    for (typename std::map<ValType, PATypeHolder>::iterator I = Map.begin(), 
+    for (typename std::map<ValType, PATypeHolder>::iterator I = Map.begin(),
          E = Map.end(); I != E; ++I)
       DerivedTypes.push_back(I->second.get());
     TypesByHash.clear();
@@ -688,7 +720,7 @@ public:
   }
 
   void RemoveFromTypesByHash(unsigned Hash, const Type *Ty) {
-    std::multimap<unsigned, PATypeHolder>::iterator I = 
+    std::multimap<unsigned, PATypeHolder>::iterator I =
       TypesByHash.lower_bound(Hash);
     while (I->second != Ty) {
       ++I;
@@ -730,7 +762,7 @@ public:
       }
 
     unsigned TypeHash = ValType::hashTypeStructure(Ty);
-    
+
     // If there are no cycles going through this node, we can do a simple,
     // efficient lookup in the map, instead of an inefficient nasty linear
     // lookup.
@@ -750,7 +782,7 @@ public:
         Ty->refineAbstractTypeTo(NewTy);
         return;
       }
-      
+
     } else {
       // Now we check to see if there is an existing entry in the table which is
       // structurally identical to the newly refined type.  If so, this type
@@ -764,7 +796,7 @@ public:
           if (TypesEqual(Ty, I->second)) {
             assert(Ty->isAbstract() && "Replacing a non-abstract type?");
             TypeClass *NewTy = cast<TypeClass>((Type*)I->second.get());
-            
+
             if (Entry == E) {
               // Find the location of Ty in the TypesByHash structure.
               while (I->second != Ty) {
@@ -779,7 +811,7 @@ public:
             return;
           }
         } else {
-          // Remember the position of 
+          // Remember the position of
           Entry = I;
         }
       }
@@ -801,14 +833,14 @@ public:
     if (Ty->isAbstract())
       Ty->PromoteAbstractToConcrete();
   }
-  
+
   void print(const char *Arg) const {
 #ifdef DEBUG_MERGE_TYPES
     std::cerr << "TypeMap<>::" << Arg << " table contents:\n";
     unsigned i = 0;
     for (typename std::map<ValType, PATypeHolder>::const_iterator I
            = Map.begin(), E = Map.end(); I != E; ++I)
-      std::cerr << " " << (++i) << ". " << (void*)I->second.get() << " " 
+      std::cerr << " " << (++i) << ". " << (void*)I->second.get() << " "
                 << *I->second.get() << "\n";
 #endif
   }
@@ -873,7 +905,7 @@ FunctionValType FunctionValType::get(const FunctionType *FT) {
 
 
 // FunctionType::get - The factory function for the FunctionType class...
-FunctionType *FunctionType::get(const Type *ReturnType, 
+FunctionType *FunctionType::get(const Type *ReturnType,
                                 const std::vector<const Type*> &Params,
                                 bool isVarArg) {
   FunctionValType VT(ReturnType, Params, isVarArg);
@@ -894,16 +926,16 @@ FunctionType *FunctionType::get(const Type *ReturnType,
 namespace llvm {
 class ArrayValType {
   const Type *ValTy;
-  unsigned Size;
+  uint64_t Size;
 public:
-  ArrayValType(const Type *val, int sz) : ValTy(val), Size(sz) {}
+  ArrayValType(const Type *val, uint64_t sz) : ValTy(val), Size(sz) {}
 
   static ArrayValType get(const ArrayType *AT) {
     return ArrayValType(AT->getElementType(), AT->getNumElements());
   }
 
   static unsigned hashTypeStructure(const ArrayType *AT) {
-    return AT->getNumElements();
+    return (unsigned)AT->getNumElements();
   }
 
   // Subclass should override this... to update self as usual
@@ -921,7 +953,7 @@ public:
 static TypeMap<ArrayValType, ArrayType> ArrayTypes;
 
 
-ArrayType *ArrayType::get(const Type *ElementType, unsigned NumElements) {
+ArrayType *ArrayType::get(const Type *ElementType, uint64_t NumElements) {
   assert(ElementType && "Can't get array of null types!");
 
   ArrayValType AVT(ElementType, NumElements);
@@ -1004,7 +1036,7 @@ public:
     ElTypes.reserve(ST->getNumElements());
     for (unsigned i = 0, e = ST->getNumElements(); i != e; ++i)
       ElTypes.push_back(ST->getElementType(i));
-    
+
     return StructValType(ElTypes);
   }
 
@@ -1078,6 +1110,10 @@ static TypeMap<PointerValType, PointerType> PointerTypes;
 
 PointerType *PointerType::get(const Type *ValueType) {
   assert(ValueType && "Can't get a pointer to <null> type!");
+  // FIXME: The sparc backend makes void pointers, which is horribly broken.
+  // "Fix" it, then reenable this assertion.
+  //assert(ValueType != Type::VoidTy &&
+  //       "Pointer to void is not valid, use sbyte* instead!");
   PointerValType PVT(ValueType);
 
   PointerType *PT = PointerTypes.get(PVT);
@@ -1115,12 +1151,12 @@ void DerivedType::removeAbstractTypeUser(AbstractTypeUser *U) const {
   assert(i < AbstractTypeUsers.size() && "Index out of range!");  // Wraparound?
 
   AbstractTypeUsers.erase(AbstractTypeUsers.begin()+i);
-      
+
 #ifdef DEBUG_MERGE_TYPES
   std::cerr << "  remAbstractTypeUser[" << (void*)this << ", "
             << *this << "][" << i << "] User = " << U << "\n";
 #endif
-    
+
   if (AbstractTypeUsers.empty() && getRefCount() == 0 && isAbstract()) {
 #ifdef DEBUG_MERGE_TYPES
     std::cerr << "DELETEing unused abstract type: <" << *this
@@ -1217,7 +1253,7 @@ void DerivedType::notifyUsesThatTypeBecameConcrete() {
            "AbstractTypeUser did not remove itself from the use list!");
   }
 }
-  
+
 
 
 
@@ -1327,7 +1363,7 @@ void Type::clearAllTypeMaps() {
   ArrayTypes.clear(DerivedTypes);
   PackedTypes.clear(DerivedTypes);
 
-  for(std::vector<Type *>::iterator I = DerivedTypes.begin(), 
+  for(std::vector<Type *>::iterator I = DerivedTypes.begin(),
       E = DerivedTypes.end(); I != E; ++I)
     (*I)->ContainedTys.clear();
   for(std::vector<Type *>::iterator I = DerivedTypes.begin(),
