@@ -1,10 +1,10 @@
 //===-- Verifier.cpp - Implement the Module Verifier -------------*- C++ -*-==//
-// 
+//
 //                     The LLVM Compiler Infrastructure
 //
 // This file was developed by the LLVM research group and is distributed under
 // the University of Illinois Open Source License. See LICENSE.TXT for details.
-// 
+//
 //===----------------------------------------------------------------------===//
 //
 // This file defines the function verifier interface, that can be used for some
@@ -41,6 +41,7 @@
 
 #include "llvm/Analysis/Verifier.h"
 #include "llvm/Assembly/Writer.h"
+#include "llvm/CallingConv.h"
 #include "llvm/Constants.h"
 #include "llvm/Pass.h"
 #include "llvm/Module.h"
@@ -76,19 +77,19 @@ namespace {  // Anonymous namespace for class
     /// an instruction in the same block.
     std::set<Instruction*> InstsInThisBlock;
 
-    Verifier() 
+    Verifier()
         : Broken(false), RealPass(true), action(AbortProcessAction),
-          DS(0), msgs( std::ios_base::app | std::ios_base::out ) {}
+          DS(0), msgs( std::ios::app | std::ios::out ) {}
     Verifier( VerifierFailureAction ctn )
-        : Broken(false), RealPass(true), action(ctn), DS(0), 
-          msgs( std::ios_base::app | std::ios_base::out ) {}
-    Verifier(bool AB ) 
-        : Broken(false), RealPass(true), 
-          action( AB ? AbortProcessAction : PrintMessageAction), DS(0), 
-          msgs( std::ios_base::app | std::ios_base::out ) {}
-    Verifier(DominatorSet &ds) 
+        : Broken(false), RealPass(true), action(ctn), DS(0),
+          msgs( std::ios::app | std::ios::out ) {}
+    Verifier(bool AB )
+        : Broken(false), RealPass(true),
+          action( AB ? AbortProcessAction : PrintMessageAction), DS(0),
+          msgs( std::ios::app | std::ios::out ) {}
+    Verifier(DominatorSet &ds)
       : Broken(false), RealPass(false), action(PrintMessageAction),
-        DS(&ds), msgs( std::ios_base::app | std::ios_base::out ) {}
+        DS(&ds), msgs( std::ios::app | std::ios::out ) {}
 
 
     bool doInitialization(Module &M) {
@@ -127,8 +128,8 @@ namespace {  // Anonymous namespace for class
         if (I->isExternal()) visitFunction(*I);
       }
 
-      for (Module::giterator I = M.gbegin(), E = M.gend(); I != E; ++I)
-        visitGlobalValue(*I);
+      for (Module::global_iterator I = M.global_begin(), E = M.global_end(); I != E; ++I)
+        visitGlobalVariable(*I);
 
       // If the module is broken, abort at this time.
       abortIfBroken();
@@ -171,12 +172,12 @@ namespace {  // Anonymous namespace for class
     // Verification methods...
     void verifySymbolTable(SymbolTable &ST);
     void visitGlobalValue(GlobalValue &GV);
+    void visitGlobalVariable(GlobalVariable &GV);
     void visitFunction(Function &F);
     void visitBasicBlock(BasicBlock &BB);
     void visitPHINode(PHINode &PN);
     void visitBinaryOperator(BinaryOperator &B);
     void visitShiftInst(ShiftInst &SI);
-    void visitVANextInst(VANextInst &VAN) { visitInstruction(VAN); }
     void visitVAArgInst(VAArgInst &VAA) { visitInstruction(VAA); }
     void visitCallInst(CallInst &CI);
     void visitGetElementPtrInst(GetElementPtrInst &GEP);
@@ -222,7 +223,7 @@ namespace {  // Anonymous namespace for class
       Broken = true;
     }
 
-    void CheckFailed( const std::string& Message, const Value* V1, 
+    void CheckFailed( const std::string& Message, const Value* V1,
                       const Type* T2, const Value* V3 = 0 ) {
       msgs << Message << "\n";
       WriteValue(V1);
@@ -262,12 +263,22 @@ void Verifier::visitGlobalValue(GlobalValue &GV) {
   }
 }
 
+void Verifier::visitGlobalVariable(GlobalVariable &GV) {
+  if (GV.hasInitializer())
+    Assert1(GV.getInitializer()->getType() == GV.getType()->getElementType(),
+            "Global variable initializer type does not match global "
+            "variable type!", &GV);
+
+  visitGlobalValue(GV);
+}
+
+
 // verifySymbolTable - Verify that a function or module symbol table is ok
 //
 void Verifier::verifySymbolTable(SymbolTable &ST) {
 
   // Loop over all of the values in all type planes in the symbol table.
-  for (SymbolTable::plane_const_iterator PI = ST.plane_begin(), 
+  for (SymbolTable::plane_const_iterator PI = ST.plane_begin(),
        PE = ST.plane_end(); PI != PE; ++PI)
     for (SymbolTable::value_const_iterator VI = PI->second.begin(),
          VE = PI->second.end(); VI != VE; ++VI) {
@@ -283,7 +294,10 @@ void Verifier::verifySymbolTable(SymbolTable &ST) {
 // visitFunction - Verify that a function is ok.
 //
 void Verifier::visitFunction(Function &F) {
-  // Check function arguments...
+  Assert1(!F.isVarArg() || F.getCallingConv() == CallingConv::C,
+          "Varargs functions must have C calling conventions!", &F);
+
+  // Check function arguments.
   const FunctionType *FT = F.getFunctionType();
   unsigned NumArgs = F.getArgumentList().size();
 
@@ -296,12 +310,12 @@ void Verifier::visitFunction(Function &F) {
 
   // Check that the argument values match the function type for this function...
   unsigned i = 0;
-  for (Function::aiterator I = F.abegin(), E = F.aend(); I != E; ++I, ++i) {
+  for (Function::arg_iterator I = F.arg_begin(), E = F.arg_end(); I != E; ++I, ++i) {
     Assert2(I->getType() == FT->getParamType(i),
             "Argument value does not match function argument type!",
             I, FT->getParamType(i));
     // Make sure no aggregates are passed by value.
-    Assert1(I->getType()->isFirstClassType(), 
+    Assert1(I->getType()->isFirstClassType(),
             "Functions cannot take aggregates as arguments by value!", I);
    }
 
@@ -329,7 +343,7 @@ void Verifier::visitBasicBlock(BasicBlock &BB) {
   if (isa<PHINode>(BB.front())) {
     std::vector<BasicBlock*> Preds(pred_begin(&BB), pred_end(&BB));
     std::sort(Preds.begin(), Preds.end());
-    PHINode *PN; 
+    PHINode *PN;
     for (BasicBlock::iterator I = BB.begin(); (PN = dyn_cast<PHINode>(I));++I) {
 
       // Ensure that PHI nodes have at least one entry!
@@ -339,7 +353,7 @@ void Verifier::visitBasicBlock(BasicBlock &BB) {
       Assert1(PN->getNumIncomingValues() == Preds.size(),
               "PHINode should have one entry for each predecessor of its "
               "parent basic block!", PN);
-      
+
       // Get and sort all incoming values in the PHI node...
       std::vector<std::pair<BasicBlock*, Value*> > Values;
       Values.reserve(PN->getNumIncomingValues());
@@ -347,7 +361,7 @@ void Verifier::visitBasicBlock(BasicBlock &BB) {
         Values.push_back(std::make_pair(PN->getIncomingBlock(i),
                                         PN->getIncomingValue(i)));
       std::sort(Values.begin(), Values.end());
-      
+
       for (unsigned i = 0, e = Values.size(); i != e; ++i) {
         // Check to make sure that if there is more than one entry for a
         // particular basic block in this PHI node, that the incoming values are
@@ -358,12 +372,12 @@ void Verifier::visitBasicBlock(BasicBlock &BB) {
                 "PHI node has multiple entries for the same basic block with "
                 "different incoming values!", PN, Values[i].first,
                 Values[i].second, Values[i-1].second);
-        
+
         // Check to make sure that the predecessors and PHI node entries are
         // matched up.
         Assert3(Values[i].first == Preds[i],
                 "PHI node entries do not match predecessors!", PN,
-                Values[i].first, Preds[i]);        
+                Values[i].first, Preds[i]);
       }
     }
   }
@@ -498,11 +512,11 @@ void Verifier::visitBinaryOperator(BinaryOperator &B) {
     Assert1(B.getType() == B.getOperand(0)->getType(),
             "Arithmetic operators must have same type for operands and result!",
             &B);
-    Assert1(B.getType()->isInteger() || B.getType()->isFloatingPoint() || 
+    Assert1(B.getType()->isInteger() || B.getType()->isFloatingPoint() ||
             isa<PackedType>(B.getType()),
             "Arithmetic operators must have integer, fp, or packed type!", &B);
   }
-  
+
   visitInstruction(B);
 }
 
@@ -546,7 +560,7 @@ void Verifier::visitStoreInst(StoreInst &SI) {
 /// verifyInstruction - Verify that an instruction is well formed.
 ///
 void Verifier::visitInstruction(Instruction &I) {
-  BasicBlock *BB = I.getParent();  
+  BasicBlock *BB = I.getParent();
   Assert1(BB, "Instruction not embedded in basic block!", &I);
 
   if (!isa<PHINode>(I)) {   // Check that non-phi nodes are not self referential
@@ -581,6 +595,7 @@ void Verifier::visitInstruction(Instruction &I) {
   for (unsigned i = 0, e = I.getNumOperands(); i != e; ++i) {
     // Check to make sure that the "address of" an intrinsic function is never
     // taken.
+    Assert1(I.getOperand(i) != 0, "Instruction has null operand!", &I);
     if (Function *F = dyn_cast<Function>(I.getOperand(i))) {
       Assert1(!F->isIntrinsic() || (i == 0 && isa<CallInst>(I)),
               "Cannot take the address of an intrinsic!", &I);
@@ -639,10 +654,10 @@ void Verifier::visitIntrinsicFunctionCall(Intrinsic::ID ID, CallInst &CI) {
     Assert1(CI.getParent()->getParent()->getFunctionType()->isVarArg(),
             "llvm.va_start intrinsic may only occur in function with variable"
             " args!", &CI);
-    NumArgs = 0;
+    NumArgs = 1;
     break;
   case Intrinsic::vaend:          NumArgs = 1; break;
-  case Intrinsic::vacopy:         NumArgs = 1; break;
+  case Intrinsic::vacopy:         NumArgs = 2; break;
 
   case Intrinsic::returnaddress:
   case Intrinsic::frameaddress:
@@ -711,6 +726,28 @@ void Verifier::visitIntrinsicFunctionCall(Intrinsic::ID ID, CallInst &CI) {
     NumArgs = 2;
     break;
 
+  case Intrinsic::ctpop:
+  case Intrinsic::ctlz:
+  case Intrinsic::cttz:
+    Assert1(FT->getNumParams() == 1,
+            "Illegal # arguments for intrinsic function!", IF);
+    Assert1(FT->getReturnType() == FT->getParamType(0),
+            "Return type does not match source type", IF);
+    Assert1(FT->getParamType(0)->isIntegral(),
+            "Argument must be of an int type!", IF);
+    NumArgs = 1;
+    break;
+
+  case Intrinsic::sqrt:
+    Assert1(FT->getNumParams() == 1,
+            "Illegal # arguments for intrinsic function!", IF);
+    Assert1(FT->getParamType(0)->isFloatingPoint(),
+            "Argument is not a floating point type!", IF);
+    Assert1(FT->getReturnType() == FT->getParamType(0),
+            "Return type is not the same as argument type!", IF);
+    NumArgs = 1;
+    break;
+
   case Intrinsic::setjmp:          NumArgs = 1; break;
   case Intrinsic::longjmp:         NumArgs = 2; break;
   case Intrinsic::sigsetjmp:       NumArgs = 2; break;
@@ -735,8 +772,15 @@ void Verifier::visitIntrinsicFunctionCall(Intrinsic::ID ID, CallInst &CI) {
   case Intrinsic::memcpy:          NumArgs = 4; break;
   case Intrinsic::memmove:         NumArgs = 4; break;
   case Intrinsic::memset:          NumArgs = 4; break;
- 
-  case Intrinsic::not_intrinsic: 
+
+  case Intrinsic::prefetch:        NumArgs = 3; break;
+  case Intrinsic::pcmarker:
+    NumArgs = 1;
+    Assert1(isa<Constant>(CI.getOperand(1)),
+            "First argument to llvm.pcmarker must be a constant!", &CI);
+    break;
+
+  case Intrinsic::not_intrinsic:
     assert(0 && "Invalid intrinsic!"); NumArgs = 0; break;
   }
 
@@ -755,11 +799,11 @@ FunctionPass *llvm::createVerifierPass(VerifierFailureAction action) {
 }
 
 
-// verifyFunction - Create 
+// verifyFunction - Create
 bool llvm::verifyFunction(const Function &f, VerifierFailureAction action) {
   Function &F = const_cast<Function&>(f);
   assert(!F.isExternal() && "Cannot verify external functions");
-  
+
   FunctionPassManager FPM(new ExistingModuleProvider(F.getParent()));
   Verifier *V = new Verifier(action);
   FPM.add(V);
