@@ -23,7 +23,7 @@
 #  -parallel        Run two parallel jobs with GNU Make.
 #  -release         Build an LLVM Release version
 #  -pedantic        Enable additional GCC warnings to detect possible errors.
-#  -enable-linscan  Enable linearscan tests
+#  -enable-llcbeta  Enable testing of beta features in llc.
 #  -disable-llc     Disable LLC tests in the nightly tester.
 #  -disable-jit     Disable JIT tests in the nightly tester.
 #  -verbose         Turn on some debug output
@@ -34,6 +34,15 @@
 #  -gnuplotscript   Next argument specifies gnuplot script to use
 #  -templatefile    Next argument specifies template file to use
 #  -gccpath         Path to gcc/g++ used to build LLVM
+#  -cvstag          Check out a specific CVS tag to build LLVM (useful for
+#                   testing release branches)
+#  -target          Specify the target triplet
+#
+#  ---------------- Options to configure llvm-test ----------------------------
+#  -spec2000path    Path to the benchspec directory in the SPEC 2000 distro
+#  -spec95path      Path to the benchspec directory in the SPEC 95 distro.
+#  -povraypath      Path to the povray sources
+#  -namdpath        Path to the namd sources
 #
 # CVSROOT is the CVS repository from which the tree will be checked out,
 #  specified either in the full :method:user@host:/dir syntax, or
@@ -66,7 +75,7 @@ my $WebDir     = $ENV{'WEBDIR'};
 @TIME = localtime;
 my $DATE = sprintf "%4d-%02d-%02d", $TIME[5]+1900, $TIME[4]+1, $TIME[3];
 my $DateString = strftime "%B %d, %Y", localtime;
-my $TestStartTime = gmtime;
+my $TestStartTime = gmtime() . "GMT<br>" . localtime() . " (local)";
 
 # Command line argument settings...
 my $NOCHECKOUT = 0;
@@ -79,9 +88,12 @@ my $PROGTESTOPTS = "";
 my $VERBOSE = 0;
 my $DEBUG = 0;
 my $CONFIGUREARGS = "";
+my $CVSCOOPT = "-APR";
 my $NICE = "";
 my $NODEJAGNU = 0;
 
+my $LLVMTESTCONFIGARGS = "";
+
 sub ReadFile {
   if (open (FILE, $_[0])) {
     undef $/;
@@ -198,9 +210,11 @@ sub GetDejagnuTestResults { # (filename, log)
   my @lines;
   my $firstline;
   $/ = "\n"; #Make sure we're going line at a time.
+
+  print "DEJAGNU TEST RESULTS:\n";
+
   if (open SRCHFILE, $filename) {
     # Process test results
-    push(@lines,"<h3>UNEXPECTED TEST RESULTS</h3><ol><li>\n");
     my $first_list = 1;
     my $should_break = 1;
     my $nocopy = 0;
@@ -211,20 +225,30 @@ sub GetDejagnuTestResults { # (filename, log)
         if ( m/^XPASS:/ || m/^FAIL:/ ) {
           $nocopy = 0;
           if ( $first_list ) {
+            push(@lines, "<h3>UNEXPECTED TEST RESULTS</h3><ol><li>\n");
             $first_list = 0;
             $should_break = 1;
-            push(@lines,"<b>$_</b><br/>\n");
+            push(@lines, "<b>$_</b><br/>\n");
+            print "  $_\n";
           } else {
-            push(@lines,"</li><li><b>$_</b><br/>\n");
+            push(@lines, "</li><li><b>$_</b><br/>\n");
+            print "  $_\n";
           }
         } elsif ( m/Summary/ ) {
-          if ( $first_list ) { push(@lines,"<b>PERFECT!</b>"); }
-          push(@lines,"</li></ol><h3>STATISTICS</h3><pre>\n");
+          if ( $first_list ) {
+            push(@lines, "<b>PERFECT!</b>"); 
+            print "  PERFECT!\n";
+          } else {
+            push(@lines, "</li></ol>\n");
+          }
+          push(@lines, "<h3>STATISTICS</h3><pre>\n");
+          print "\nDEJAGNU STATISTICS:\n";
           $should_break = 0;
           $nocopy = 0;
           $readingsum = 1;
         } elsif ( $readingsum ) {
           push(@lines,"$_\n");
+          print "  $_\n";
         }
       }
     }
@@ -232,7 +256,7 @@ sub GetDejagnuTestResults { # (filename, log)
   push(@lines, "</pre>\n");
   close SRCHFILE;
 
-  my $content = join("",@lines);
+  my $content = join("", @lines);
   return "$content</li></ol>\n";
 }
 
@@ -257,10 +281,10 @@ while (scalar(@ARGV) and ($_ = $ARGV[0], /^[-+]/)) {
   if (/^-parallel$/)       { $MAKEOPTS = "$MAKEOPTS -j2 -l3.0"; next; }
   if (/^-release$/)        { $MAKEOPTS = "$MAKEOPTS ENABLE_OPTIMIZED=1"; next; }
   if (/^-pedantic$/)       { 
-      $MAKEOPTS   = "$MAKEOPTS CompileOptimizeOpts='-O3 -DNDEBUG -finline-functions -Wpointer-arith -Wcast-align -Wno-deprecated -Wold-style-cast -Wabi -Woverloaded-virtual -ffor-scope'"; 
+      $MAKEOPTS = "$MAKEOPTS CompileOptimizeOpts='-O3 -DNDEBUG -finline-functions -Wpointer-arith -Wcast-align -Wno-deprecated -Wold-style-cast -Wabi -Woverloaded-virtual -ffor-scope'"; 
       next; 
   }
-  if (/^-enable-linscan$/) { $PROGTESTOPTS .= " ENABLE_LINEARSCAN=1"; next; }
+  if (/^-enable-llcbeta$/) { $PROGTESTOPTS .= " ENABLE_LLCBETA=1"; next; }
   if (/^-disable-llc$/)    { $PROGTESTOPTS .= " DISABLE_LLC=1";
                              $CONFIGUREARGS .= " --disable-llc_diffs"; next; }
   if (/^-disable-jit$/)    { $PROGTESTOPTS .= " DISABLE_JIT=1";
@@ -276,9 +300,24 @@ while (scalar(@ARGV) and ($_ = $ARGV[0], /^[-+]/)) {
   if (/^-gccpath/)         { 
     $CONFIGUREARGS .= " CC=$ARGV[0]/gcc CXX=$ARGV[0]/g++"; shift; next; 
   }
+  if (/^-cvstag/)          { $CVSCOOPT .= " -r $ARGV[0]"; shift; next; }
+  if (/^-target/)          {
+    $CONFIGUREARGS .= " --target=$ARGV[0]"; shift; next;
+  }
   if (/^-noexternals$/)    { $NOEXTERNALS = 1; next; }
-  if(/^-nodejagnu$/) { $NODEJAGNU = 1; next; }
-
+  if (/^-nodejagnu$/)      { $NODEJAGNU = 1; next; }
+  if (/^-spec2000path$/)   {
+    $LLVMTESTCONFIGARGS .= " --enable-spec2000=$ARGV[0]"; shift; next;
+  }
+  if (/^-spec95path$/)     {
+    $LLVMTESTCONFIGARGS .= " --enable-spec95=$ARGV[0]"; shift; next;
+  }
+  if (/^-povraypath$/)     {
+    $LLVMTESTCONFIGARGS .= " --enable-povray=$ARGV[0]"; shift; next;
+  }
+  if (/^-namdpath$/)       {
+    $LLVMTESTCONFIGARGS .= " --enable-namd=$ARGV[0]"; shift; next;
+  }
   print "Unknown option: $_ : ignoring!\n";
 }
 
@@ -345,12 +384,14 @@ ChangeDir( $BuildDir, "CVS checkout directory" );
 #
 # Check out the llvm tree, saving CVS messages to the cvs log...
 #
-$CVSOPT = "";
-$CVSOPT = "-z3" if $CVSRootDir =~ /^:ext:/; # Use compression if going over ssh.
+my $CVSOPT = "";
+# Use compression if going over ssh.
+$CVSOPT = "-z3" if $CVSRootDir =~ /^:ext:/;
+my $CVSCMD = "$NICE cvs $CVSOPT -d $CVSRootDir co $CVSCOOPT";
 if (!$NOCHECKOUT) {
   if ( $VERBOSE ) { print "CHECKOUT STAGE\n"; }
-  system "( time -p $NICE cvs $CVSOPT -d $CVSRootDir co -APR llvm; cd llvm/projects ; " .
-     "$NICE cvs $CVSOPT -d $CVSRootDir co -APR llvm-test ) > $CVSLog 2>&1";
+  system "( time -p $CVSCMD llvm; cd llvm/projects ; " .
+     "$CVSCMD llvm-test ) > $CVSLog 2>&1";
   ChangeDir( $BuildDir , "CVS Checkout directory") ;
 }
 
@@ -388,7 +429,8 @@ $LOC = `utils/countloc.sh`;
 #
 if (!$NOCHECKOUT) {
   if ( $VERBOSE ) { print "CONFIGURE STAGE\n"; }
-  system "(time -p $NICE ./configure $CONFIGUREARGS --enable-spec --with-objroot=.) > $BuildLog 2>&1";
+  my $EXTRAFLAGS = "--enable-spec --with-objroot=.$LLVMTESTCONFIGARGS";
+  system "(time -p $NICE ./configure $CONFIGUREARGS $EXTRAFLAGS) > $BuildLog 2>&1";
 
   if ( $VERBOSE ) { print "BUILD STAGE\n"; }
   # Build the entire tree, capturing the output into $BuildLog
@@ -402,7 +444,7 @@ if (!$NOCHECKOUT) {
 my @Linked = split '\n', `grep Linking $BuildLog`;
 my $NumExecutables = scalar(grep(/executable/, @Linked));
 my $NumLibraries   = scalar(grep(!/executable/, @Linked));
-my $NumObjects     = `grep '^Compiling' $BuildLog | wc -l` + 0;
+my $NumObjects     = `grep ']\: Compiling ' $BuildLog | wc -l` + 0;
 
 my $ConfigTimeU = GetRegexNum "^user", 0, "([0-9.]+)", "$BuildLog";
 my $ConfigTimeS = GetRegexNum "^sys", 0, "([0-9.]+)", "$BuildLog";
@@ -414,11 +456,12 @@ my $BuildTimeS = GetRegexNum "^sys", 1, "([0-9.]+)", "$BuildLog";
 my $BuildTime  = $BuildTimeU+$BuildTimeS;  # BuildTime = User+System
 my $BuildWallTime = GetRegexNum "^real", 1, "([0-9.]+)","$BuildLog";
 
-my $BuildError = "";
+my $BuildError = 0, $BuildStatus = "OK";
 if (`grep '^gmake[^:]*: .*Error' $BuildLog | wc -l` + 0 ||
     `grep '^gmake: \*\*\*.*Stop.' $BuildLog | wc -l`+0) {
-  $BuildError = "<h3><font color='red'>Build error: compilation " .
+  $BuildStatus = "<h3><font color='red'>error: compilation " .
                 "<a href=\"$DATE-Build-Log.txt\">aborted</a></font></h3>";
+  $BuildError = 1;
   if ($VERBOSE) { print "BUILD ERROR\n"; }
 }
 
@@ -445,6 +488,7 @@ if(!$NODEJAGNU) {
   CopyFile("test/testrun.sum", $DejagnuSum);
 
   $DejagnuTestResults = GetDejagnuTestResults($DejagnuSum, $DejagnuLog);
+
 } else {
   $DejagnuTestResults = "Skipped by user choice.";
   $DejagnuTime     = "0.0";
@@ -532,8 +576,8 @@ foreach $File (@CVSHistory) {
   }
 }
 
-my $UserCommitList = join "\n", keys %UsersCommitted;
-my $UserUpdateList = join "\n", keys %UsersUpdated;
+my $UserCommitList = join "\n", sort keys %UsersCommitted;
+my $UserUpdateList = join "\n", sort keys %UsersUpdated;
 my $AddedFilesList = AddPreTag join "\n", sort keys %AddedFiles;
 my $ModifiedFilesList = AddPreTag join "\n", sort keys %ModifiedFiles;
 my $RemovedFilesList = AddPreTag join "\n", sort keys %RemovedFiles;
@@ -587,7 +631,7 @@ sub TestDirectory {
 }
 
 # If we built the tree successfully, run the nightly programs tests...
-if ($BuildError eq "") {
+if (!$BuildError) {
   if ( $VERBOSE ) {
     print "SingleSource TEST STAGE\n";
   }
@@ -661,7 +705,7 @@ if ($TestError) {
 
 # If we built the tree successfully, runs of the Olden suite with
 # LARGE_PROBLEM_SIZE on so that we can get some "running" statistics.
-if ($BuildError eq "") {
+if (!$BuildError) {
   if ( $VERBOSE ) { print "OLDEN TEST SUITE STAGE\n"; }
   my ($NATTime, $CBETime, $LLCTime, $JITTime, $OptTime, $BytecodeSize,
       $MachCodeSize) = ("","","","","","","");
@@ -688,15 +732,14 @@ if ($BuildError eq "") {
 
   # Loop over all of the records, summarizing them into rows for the running
   # totals file.
-  my $WallTimeRE = "[A-Za-z0-9.: ]+\\(([0-9.]+) wall clock";
+  my $WallTimeRE = "Time: ([0-9.]+) seconds \\([0-9.]+ wall clock";
   foreach $Rec (@Records) {
     my $rNATTime = GetRegex 'TEST-RESULT-nat-time: program\s*([.0-9m]+)', $Rec;
     my $rCBETime = GetRegex 'TEST-RESULT-cbe-time: program\s*([.0-9m]+)', $Rec;
     my $rLLCTime = GetRegex 'TEST-RESULT-llc-time: program\s*([.0-9m]+)', $Rec;
     my $rJITTime = GetRegex 'TEST-RESULT-jit-time: program\s*([.0-9m]+)', $Rec;
-    my $rOptTime = GetRegex "TEST-RESULT-compile: $WallTimeRE", $Rec;
+    my $rOptTime = GetRegex "TEST-RESULT-compile: .*$WallTimeRE", $Rec;
     my $rBytecodeSize = GetRegex 'TEST-RESULT-compile: *([0-9]+)', $Rec;
-    my $rMachCodeSize = GetRegex 'TEST-RESULT-jit-machcode: *([0-9]+).*bytes of machine code', $Rec;
 
     $NATTime .= " " . FormatTime($rNATTime);
     $CBETime .= " " . FormatTime($rCBETime);
@@ -704,7 +747,6 @@ if ($BuildError eq "") {
     $JITTime .= " " . FormatTime($rJITTime);
     $OptTime .= " $rOptTime";
     $BytecodeSize .= " $rBytecodeSize";
-    $MachCodeSize .= " $rMachCodeSize";
   }
 
   # Now that we have all of the numbers we want, add them to the running totals
@@ -715,7 +757,6 @@ if ($BuildError eq "") {
   AddRecord($JITTime, "running_Olden_jit_time.txt");
   AddRecord($OptTime, "running_Olden_opt_time.txt");
   AddRecord($BytecodeSize, "running_Olden_bytecode.txt");
-  AddRecord($MachCodeSize, "running_Olden_machcode.txt");
 
   system "gzip -f $OldenTestsLog";
 }
@@ -741,7 +782,7 @@ ChangeDir( $WebDir, "Web Directory" );
 # Make sure we don't get errors running the nightly tester the first time
 # because of files that don't exist.
 Touch ('running_build_time.txt', 'running_Olden_llc_time.txt',
-       'running_loc.txt', 'running_Olden_machcode.txt',
+       'running_loc.txt', 
        'running_Olden_bytecode.txt', 'running_Olden_nat_time.txt',
        'running_Olden_cbe_time.txt', 'running_Olden_opt_time.txt',
        'running_Olden_jit_time.txt');
@@ -765,6 +806,18 @@ system ("$GNUPLOT", $PlotScriptFilename);
 #
 system ( "$NICE rm -rf $BuildDir") if (!$NOCHECKOUT and !$NOREMOVE);
 
+print "\nUSERS WHO COMMITTED:\n  " . (join "\n  ", sort keys %UsersCommitted) . "\n"
+  if (scalar %UsersCommitted);
+
+print "\nADDED FILES:\n  " . (join "\n  ", sort keys %AddedFiles) . "\n"
+  if (scalar %AddedFiles);
+
+print "\nCHANGED FILES:\n  " . (join "\n  ", sort keys %ModifiedFiles) . "\n"
+  if (scalar %ModifiedFiles);
+
+print "\nREMOVED FILES:\n  " . (join "\n  ", sort keys %RemovedFiles) . "\n"
+  if (scalar %RemovedFiles);
+
 #
 # Print out information...
 #
@@ -776,12 +829,6 @@ if ($VERBOSE) {
   print "Libraries/Executables/Objects built: $NumLibraries/$NumExecutables/$NumObjects\n";
 
   print "WARNINGS:\n  $WarningsList\n";
-
-  print "Users committed: $UserCommitList\n";
-  print "Added Files: \n  $AddedFilesList\n";
-  print "Modified Files: \n  $ModifiedFilesList\n";
-  print "Removed Files: \n  $RemovedFilesList\n";
-
   print "Previous Days =\n  $PrevDaysList\n";
 }
 
@@ -795,7 +842,8 @@ if ( $VERBOSE ) {
 }
 # Main HTML file...
 my $Output;
-my $TestFinishTime = gmtime;
+my $TestFinishTime = gmtime() . " GMT<br>" . localtime() . " (local)";
+
 my $TestPlatform = `uname -a`;
 eval "\$Output = <<ENDOFFILE;$TemplateContents\nENDOFFILE\n";
 WriteFile "$DATE.html", $Output;
